/*
  $Id: pinned_mem.c,v 1.2 2003/03/25 15:57:49 roland Exp $
  Allocated memory covered by the pinned TLB entries

  Jan 8, 2003 - Roland Dreier <roland@topspin.com>
*/

#include <linux/pagemap.h>
#include <linux/vmalloc.h>
#include <linux/kernel.h>
#include <linux/module.h>

static inline int nextorder(unsigned int x)
{
	int c = -PAGE_SHIFT;
	while (x) {
		x >>= 1; 
		c++; 
	}
	if (c < 0) 
		c = 0;
	return c; 
}  

void *pinned_map(unsigned long size) {
	struct page *p, *w; 
	int order = nextorder(size); 

	p = alloc_pages(GFP_KERNEL | GFP_DMA, order);
	if (p) {
		struct page *end = p + (1UL << order); 
		for (w = p+1; w < end; ++w) 
			set_page_count(w, 1); 
		for (w = p + (size>>PAGE_SHIFT)+1; w < end; ++w) 	      
			__free_pages(w, 0); 
		return (void *) page_address(p); 
	} 

	return NULL;
}

void pinned_unmap(void *addr, size_t size) {
	int sz = size;
	struct page *w; 
	unsigned long mptr = (unsigned long) addr; 

	if (mptr >= VMALLOC_START && mptr + sz <= VMALLOC_END) { 
		vfree(addr); 
		return;
	} 

	w = virt_to_page(addr); 
	for (; sz > 0; sz -= PAGE_SIZE, ++w) { 
		if (atomic_read(&w->count) != 1) 
			BUG(); 
		__free_pages(w, 0); 
	}
}
