/***************************************************************************
*
*   Module:             
*
*   Filename:           270sc_i2c_slave.c
*
*   Author:             Brian Tran
*
*   Created:            06/03/2004
*
*   Description:
*   
*   Receive data bytes written to the i2c controller as a slave.
*
****************************************************************************
*                 Source Control System Information
*
*    $Id: 270sc_i2c_slave.c,v 1.19 2005/03/16 22:41:53 brian Exp $
*
****************************************************************************
*                       Modification History
*
*    $Log: 270sc_i2c_slave.c,v $
*    Revision 1.19  2005/03/16 22:41:53  brian
*    Bugzilla number:
*    Increase max FRU number for 270 to include optical modules. Increase the FRU id size to 7 bits
*
*    Revision 1.18  2005/02/23 18:26:25  brian
*    Bugzilla number: 5922
*    Fixed a bug in array index to post_err
*
*    Revision 1.17  2005/02/22 23:55:31  brian
*    Bugzilla number: 5922
*    Support POST results of any FRU reported by standby core
*
*    Revision 1.16  2004/10/26 20:52:27  brian
*    Bugzilla number:
*    Replace a constant with #define
*
*    Revision 1.15  2004/09/24 22:17:04  brian
*    Bugzilla number:
*    Change default status to include stateinfo == 0
*
*    Revision 1.14  2004/09/24 00:33:53  brian
*    Bugzilla number:
*    Support getting POST result
*
*    Revision 1.13  2004/07/27 17:36:36  brian
*    Bugzilla number:
*    Add command to report hw errors thru I2C
*
*    Revision 1.12  2004/07/21 01:04:25  brian
*    Bugzilla number:
*    Fix a bug in handling writing into buffer
*
*    Revision 1.11  2004/07/20 18:16:51  brian
*    Bugzilla number:
*    Increase buffer size to handle 2 concurrent masters sending commands
*
*    Revision 1.10  2004/07/16 01:39:16  brian
*    Bugzilla number:
*    Add lock to i2c status
*
*    Revision 1.9  2004/07/14 20:43:31  brian
*    Bugzilla number:
*    Fixed a bug in frustate data
*
*    Revision 1.8  2004/07/14 02:20:14  brian
*    Bugzilla number:
*    More support to sync fru state
*
*    Revision 1.7  2004/07/13 01:55:59  brian
*    Bugzilla number:
*    Fixed bug in receiving i2c command
*
*    Revision 1.6  2004/07/10 02:03:46  brian
*    Bugzilla number:
*    Support to sync fru data
*
*    Revision 1.5  2004/07/01 00:46:00  brian
*    Bugzilla number:
*    Now fpga_base_addr is a global var
*
*    Revision 1.4  2004/06/30 01:58:29  brian
*    Bugzilla number:
*    Support fpga sync between 2 cores
*
*    Revision 1.3  2004/06/09 02:02:47  brian
*    Bugzilla number:
*    Add lock in i2c sendcommand
*
*    Revision 1.2  2004/06/08 21:34:25  brian
*    Bugzilla number:
*    Allow to set status via i2c
*
*    Revision 1.1  2004/06/04 01:33:04  brian
*    Bugzilla number:
*    Support for i2c slave in 270
*
*
***************************************************************************
*                       (c) COPYRIGHT 2000, 2001, 2002
*                           TopSpin Communications, Inc.
*                           Palo Alto, CA
**************************************************************************/
#ifdef RCSID
static const char rcsid[] = "$Id: 270sc_i2c_slave.c,v 1.19 2005/03/16 22:41:53 brian Exp $";
#endif


#include <linux/config.h>
#include <linux/version.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <asm/uaccess.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/machdep.h>
#include <asm/ocp.h>
#include <linux/i2c-errcode.h>
#include <topspin_types.h>
#include <fpga.h>
#include <system_interrupt_code.h>
#include "270sc_sysevent.h"
#include "ds1672_rtc.h"
#include "dcr.h"
#include "gateway_i2c_serial.h"

extern int phy_iic_xfer(void *vaddr, unsigned short dev_addr, int access, char *buf, int count, int controller);
extern struct semaphore i2c_lock;

extern volatile int hardware_err_sync_int_active;
extern volatile int err_fru_id[TS270_MAX_NUM_FRU];
extern unsigned char post_err[TS270_MAX_NUM_FRU][POST_RESULT_LEN];
extern wait_queue_head_t sysevent_waitq;


/* Function Prototypes */
static ssize_t i2cslave_read(struct file *file, char *buf, size_t count, loff_t *offset);
static ssize_t i2cslave_write(struct file *file, const char *buf, size_t count, loff_t *offset);


/* forward declarations */
int i2c_serial_slave_write_handler(unsigned char slave_status);
int i2c_serial_slave_read_handler(unsigned char slave_status);
int i2c_serial_init(void);
void i2cslave_cleanup_module(void);



/* File Operation Table */
static struct file_operations char_fops = {
    owner: THIS_MODULE,
    read: i2cslave_read,
    write: i2cslave_write,
};

#define TS270_I2CSLAVE_BUFSIZE		256

static wait_queue_head_t i2cslave_waitq;
static unsigned char databyte1[TS270_I2CSLAVE_BUFSIZE];
static unsigned char databyte2[TS270_I2CSLAVE_BUFSIZE];
static int current_pointer;
static unsigned char *current_read;
static unsigned char *current_write;
static unsigned char frustate[TS270_MAX_NUM_FRU]; 
static unsigned char ts270_i2c_status[POST_RESULT_LEN + 1];
static unsigned char current_status_pointer;
static struct semaphore i2c_sendcommamd_lock;
static struct semaphore i2c_status_lock;

static void *i2c0_base_addr;
static void *i2c1_base_addr;
extern void *ts270_fpga_base_addr;


#define I2CSLAVE_MAJOR      249 /* experimental range */

static unsigned char get_muxselect(int fru_location)
{
    unsigned char muxselect;

    switch (fru_location) {
        case SC_270_SW_NODE1_SLOT_NUM:
                                        muxselect = TS270_FPGA_MUX_SELECT_SLOT9;
                                        break;
        case SC_270_SW_NODE2_SLOT_NUM:
                                        muxselect = TS270_FPGA_MUX_SELECT_SLOT10;
                                        break;
        case SC_270_SW_NODE3_SLOT_NUM:
                                        muxselect = TS270_FPGA_MUX_SELECT_SLOT13;
                                        break;
        case SC_270_SW_NODE4_SLOT_NUM:
                                        muxselect = TS270_FPGA_MUX_SELECT_SLOT14;
                                        break;
        case SC_270_SW_CORE1_SLOT_NUM:
                                        muxselect = TS270_FPGA_MUX_SELECT_SLOT11;
                                        break;
        case SC_270_SW_CORE2_SLOT_NUM:
                                        muxselect = TS270_FPGA_MUX_SELECT_SLOT12;
                                        break;
        default:
                                        muxselect = 0xff;
                                        break;
    }

    return muxselect;

}




int i2c_serial_init(void)
{
    int i;

    for (i = 0; i < TS270_MAX_NUM_FRU; i ++) {
        frustate[i] = 0xff;
    }

    current_pointer = 0;
    current_read = databyte1;
    current_write = databyte1;

    /* ts270_i2c_status = TS270_I2C_CMDBUSY_STATUS_MASK << TS270_I2C_CMDBUSY_STATUS_SHIFT; */
    ts270_i2c_status[0] = TS270_I2C_DEFAULT_STATUS;		/* status byte */
    for (i = 1; i <= POST_RESULT_LEN; i ++) {
        ts270_i2c_status[i] = 0;
    }

    current_status_pointer = 0;

    init_MUTEX(&i2c_sendcommamd_lock);
    init_MUTEX(&i2c_status_lock);


    /* register char device */
    if (register_chrdev(I2CSLAVE_MAJOR, "i2cslavemodule", &char_fops)) {
        printk(KERN_ERR "Unable to register i2cslavemodule\n");
        return -EIO;
    }

    i2c0_base_addr = ioremap_nocache(I2C_CONTROLLER0_BASE, 0x12); 
    i2c1_base_addr = ioremap_nocache(I2C_CONTROLLER1_BASE, 0x12); 

    /* init wait queue */
    init_waitqueue_head(&i2cslave_waitq);


    return 0;
}


void i2cslave_cleanup_module(void)
{
    unregister_chrdev(I2CSLAVE_MAJOR, "i2cslavemodule");
    iounmap(i2c0_base_addr);
    iounmap(i2c1_base_addr);
}



/* Read routine */
static ssize_t i2cslave_read(struct file *file, char *buf, size_t count, loff_t *offset)
{
    int minor;
    struct inode *inode = file->f_dentry->d_inode;
    int len;

    minor = MINOR(inode->i_rdev);


    if (minor == I2CSLAVE_COMMAND_MINOR) {
        interruptible_sleep_on(&i2cslave_waitq);
   
        len = strlen(current_read) + 1;
        copy_to_user(buf, current_read, len);
        if (current_read == databyte1) {
            current_read = databyte2;
        }
        else {
            current_read = databyte1;
        }
        return len;
    }
    else if (minor == 0) {
        copy_to_user(buf, ts270_i2c_status, count);
        return count;
    }
    else {
        copy_to_user(buf, &frustate[minor], 1);
        return 1;
    }

}


/* Write routine
 * -------------
 */
static ssize_t i2cslave_write(struct file *file, const char *buf, size_t count, loff_t *offset)
{
    unsigned char mask;
    unsigned char shift;
    unsigned char value;
    unsigned char data;
    unsigned char tmp;
    int minor;
    struct inode *inode = file->f_dentry->d_inode;
    char mybuf[256];
    int current_count;
    unsigned char muxselect;
    unsigned char post_buf[POST_RESULT_LEN + 3];

    minor = MINOR(inode->i_rdev);

    if (minor != 0) {
        copy_from_user(mybuf, buf, count);
        muxselect = get_muxselect(minor);

        down(&i2c_sendcommamd_lock);

        for (current_count = 0; current_count < count; current_count ++) {

            /* send one byte at a time to the destination slot with muxselect  == minor number */ 
            down (&i2c_lock);
            if (phy_iic_xfer(i2c0_base_addr, (muxselect << 8) | GATEWAY_I2C_CONTROLLER_SLAVE_ADDR, WRITEACCESS, &mybuf[current_count], 1, 0) != 1) {
                up (&i2c_lock);
                up (&i2c_sendcommamd_lock);
                return current_count;
            }
            up (&i2c_lock);

        } 

        up (&i2c_sendcommamd_lock);

        return current_count;
    }

    /* if here, default minor == 0 */

    if (count < 3) {
        return count;
    }

    copy_from_user(post_buf, buf, count);

#if 0
    value = (cmd >> TS270_I2C_STATUS_VALUE_SHIFT) & 0xff; 
    shift = (cmd >> TS270_I2C_STATUS_SHIFT_SHIFT) & 0xff; 
    mask = (cmd >> TS270_I2C_STATUS_MASK_SHIFT) & 0xff; 
#endif

    mask = post_buf[0];
    shift = post_buf[1];
    value = post_buf[2];

    /* users want to change the i2c status, providing the mask, the shift, and the value */
    down(&i2c_status_lock);

    tmp = ts270_i2c_status[0];

    /* first clear the relevent bits */
    data = mask << shift;
    tmp &= ~data;

    /* then set the bits */
    data = (value & mask) << shift;
    tmp |= data;

    ts270_i2c_status[0] = tmp;

    if (count == (POST_RESULT_LEN + 3)) { 
        memcpy(&ts270_i2c_status[1], &post_buf[3], POST_RESULT_LEN);
    }

    up(&i2c_status_lock);

    return count;
}


int i2c_serial_slave_write_handler(unsigned char slave_status)
{
    volatile unsigned char received_byte;
    int i;
    unsigned char count;
    unsigned char command;
    unsigned char offset;
    unsigned char value;
    unsigned char fruid;


    /* Slave Write Need Service */
    if ((slave_status & 0x10) != 0) {
        /* make sure to empty the slave buf */
        for (i = 0; i < 4; i ++) {
            received_byte = readb((void *) ((int) i2c1_base_addr + I2C_CONTROLLER_SLAVE_DBUF));
        }
        /* ack intr */
        writeb(0x10, (void *)((int)i2c1_base_addr + I2C_CONTROLLER_EXT_CONTROL));
        return 0;
    }


    /* get the number of bytes received */
    count = readb((void *) ((int) i2c1_base_addr + I2C_CONTROLLER_TRANSFER_COUNT));
    count = (count >> 4) & 0x07;


    if (count == 1) {   /* 1-byte write for now */
        /* discard data if buffer is full */
        if (current_pointer < TS270_I2CSLAVE_BUFSIZE) {
            current_write[current_pointer] = readb((void *) ((int) i2c1_base_addr + I2C_CONTROLLER_SLAVE_DBUF));
            /* if getting a NULL, wake the reading process up */
            if (current_write[current_pointer] == 0) {
                /* change buffer */
                if (current_write == databyte1) {
                    current_write = databyte2;
                }
                else {
                    current_write = databyte1;
                }
                current_pointer = 0;
                wake_up_interruptible(&i2cslave_waitq);
            } 
            else {
                current_pointer ++;
                /* if have not seen a NULL after running out of buffer, terminate anyway */
                if (current_pointer == TS270_I2CSLAVE_BUFSIZE) {
                    current_write[current_pointer - 1] = 0;
                        if (current_write == databyte1) {
                        current_write = databyte2;
                    }
                    else {
                        current_write = databyte1;
                    }
                    current_pointer = 0;
                    wake_up_interruptible(&i2cslave_waitq);
                }
            }
        }
    }

    if (count == 3) {
        /* the first byte is the command, the second byte is the offset, the third byte is the value */
        command = readb((void *) ((int) i2c1_base_addr + I2C_CONTROLLER_SLAVE_DBUF));
        offset = readb((void *) ((int) i2c1_base_addr + I2C_CONTROLLER_SLAVE_DBUF));
        value = readb((void *) ((int) i2c1_base_addr + I2C_CONTROLLER_SLAVE_DBUF));
        if (command == TS270_FPGA_UPDATE) {
            writeb(value, (void *) ((int) ts270_fpga_base_addr + offset));
        }
        if (command == TS270_FRUSTATE_UPDATE) {
            frustate[offset] = value;
        } 
        if (command == TS270_HW_ERR_UPDATE) {
            fruid = value & SYSEVENTWRITE_FRUID_MASK;    
            err_fru_id[fruid] = (offset << 8) | value; 
            hardware_err_sync_int_active = 1;
            wake_up_interruptible(&sysevent_waitq);
        }
        if ((command & TS270_HW_ERR_UPDATE_W_POST) == TS270_HW_ERR_UPDATE_W_POST) {
            fruid = command  & SYSEVENTWRITE_FRUID_MASK;
            post_err[fruid][offset] = value;
        }
    }


    /* make sure to empty the slave buf */
    received_byte = readb((void *) ((int) i2c1_base_addr + I2C_CONTROLLER_MODE_CONTROL));
    writeb(received_byte | 0x80, (void *)((int)i2c1_base_addr + I2C_CONTROLLER_MODE_CONTROL));

    /* ack intr */
    writeb(0x20, (void *)((int)i2c1_base_addr + I2C_CONTROLLER_EXT_CONTROL));

    return 0;
}


int i2c_serial_slave_read_handler(unsigned char slave_status)
{
    unsigned char mode_control;

    if ((slave_status & 0x80) != 0) {     /* slave read complete */
        /* make sure to flush the slave buffer */
        mode_control = readb((void *) ((int) i2c1_base_addr + I2C_CONTROLLER_MODE_CONTROL));
        writeb(mode_control | 0x80, (void *)((int)i2c1_base_addr + I2C_CONTROLLER_MODE_CONTROL));

        current_status_pointer = 0;

        /* simply acknowledge the intr */
        writeb(0x80, (void *)((int)i2c1_base_addr + I2C_CONTROLLER_EXT_CONTROL));
    }


    if ((slave_status & 0x40) != 0) {     /* slave read need service */
      writeb(ts270_i2c_status[current_status_pointer], (void *)((int)i2c1_base_addr + I2C_CONTROLLER_SLAVE_DBUF));
      current_status_pointer ++;
      if (current_status_pointer > POST_RESULT_LEN) {
         current_status_pointer = 0;
      } 
      writeb(0x40, (void *)((int)i2c1_base_addr + I2C_CONTROLLER_EXT_CONTROL));
    }

    return 0;

}


