/*
 */

#include <linux/config.h>
#include <linux/delay.h>
#include <linux/ioport.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/init.h>
#include <linux/slab.h>
#include <linux/tty.h>
#include <linux/tty_driver.h>
#include <linux/tty_flip.h>
#include <linux/module.h>
#include <linux/sched.h>
#include <asm/uaccess.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/machdep.h>
#include <asm/ocp.h>

#include "ds1672_rtc.h"
#include "gateway_i2c_serial.h"

#define DRIVER_VERSION "v1.1"
#define DRIVER_AUTHOR "Brian Tran"
#define DRIVER_DESC "I2C TTY driver"

/* Module information */
MODULE_AUTHOR( DRIVER_AUTHOR );
MODULE_DESCRIPTION( DRIVER_DESC );
MODULE_LICENSE("GPL");


#define TINY_TTY_MAJOR		249	/* experimental range */
#define TINY_TTY_MINORS		14	/* for all slots */

#define PIC_PHY_ADDR				0x1f0200000LL
#define I2C_CONTROLLER1_PHY_ADDR    0x140000500LL
#define PIC_PSP_IB_INTR_HI          0x60
#define PIC_PSP_IB_INTR_LO          0x61


#define MAX_READ_SIZE		1024

extern int phy_iic_xfer(void *vaddr, unsigned short dev_addr, int access, char *buf, int count, int controller);

#ifdef PIC_MME_READY

extern int  mme_clearConsoleInterrupt(void *vaddr, int controller, int slot);
extern int  mme_enableConsoleInterrupt(void *vaddr, int controller, int slot);
extern int  mme_disableConsoleInterrupt(void *vaddr, int controller, int slot);
extern int  mme_checkConsoleInterrupt(void *vaddr, int controller, int slot, int *isInterrupt);
extern int  enable_IMxInt_L(void *vaddr, int controller, int slot);

#endif PIC_MME_READY



extern struct semaphore i2c_lock;
extern wait_queue_head_t sysevent_waitq;

int i2c_serial_slave_write_handler(unsigned char slave_status);
int i2c_serial_slave_read_handler(unsigned char slave_status);


static void i2c_serial_callback_read(void *arg);
static void i2c_serial_callback_write(void *arg);

#define MAX_I2C_SERIAL_CALLBACK_ENTRIES		256

#if 1
static struct tq_struct i2c_serial_callback_read_tq[TINY_TTY_MINORS][MAX_I2C_SERIAL_CALLBACK_ENTRIES];
static struct tq_struct i2c_serial_callback_write_tq[TINY_TTY_MINORS][MAX_I2C_SERIAL_CALLBACK_ENTRIES];
#endif


#define FPGA_MUX_SELECT_ONBOARD         0x0
#define FPGA_MUX_SELECT_SLOT1           0xc
#define FPGA_MUX_SELECT_SLOT2           0x4
#define FPGA_MUX_SELECT_SLOT3           0x8
#define FPGA_MUX_SELECT_SLOT4           0x6
#define FPGA_MUX_SELECT_SLOT5           0xa
#define FPGA_MUX_SELECT_SLOT6           0x2
#define FPGA_MUX_SELECT_SLOT7           0x9
#define FPGA_MUX_SELECT_SLOT8           0x1
#define FPGA_MUX_SELECT_SLOT9           0xe
#define FPGA_MUX_SELECT_SLOT10          0x3
#define FPGA_MUX_SELECT_SLOT11          0xd
#define FPGA_MUX_SELECT_SLOT12          0x5


unsigned char mux_select[] = { 
                        FPGA_MUX_SELECT_ONBOARD,
                        FPGA_MUX_SELECT_SLOT1,
                        FPGA_MUX_SELECT_SLOT2,
                        FPGA_MUX_SELECT_SLOT3,
                        FPGA_MUX_SELECT_SLOT4,
                        FPGA_MUX_SELECT_SLOT5,
                        FPGA_MUX_SELECT_SLOT6,
                        FPGA_MUX_SELECT_SLOT7,
                        FPGA_MUX_SELECT_SLOT8,
                        FPGA_MUX_SELECT_SLOT9,
                        FPGA_MUX_SELECT_SLOT10,
                        FPGA_MUX_SELECT_SLOT11,
                        FPGA_MUX_SELECT_SLOT12,
                    };


/* forward declaration */
int cc_i2c_serial_int_handler(int slot);


struct tiny_serial {
	struct tty_struct	*tty;		/* pointer to the tty for this device */
    int current_i2c_serial_callback_read_entry;
    int current_i2c_serial_callback_write_entry;
};

static void *pic_base_addr;
static void *i2c_base_addr;
static int			tiny_refcount;


static struct tty_driver	tiny_tty_driver;

static struct tty_struct	*tiny_tty[TINY_TTY_MINORS];
static struct termios		*tiny_termios[TINY_TTY_MINORS];
static struct termios		*tiny_termios_locked[TINY_TTY_MINORS];
static struct tiny_serial	*tiny_table[TINY_TTY_MINORS];	/* initially all NULL */



static int tiny_open (struct tty_struct *tty, struct file * filp)
{
	struct tiny_serial *tiny;
    int i;
    int min = minor(tty->device); 

	MOD_INC_USE_COUNT;

	/* initialize the pointer incase something fails */
	tty->driver_data = NULL;

	/* get the serial object associated with this tty pointer */
	tiny = tiny_table[min];
	if (tiny == NULL) {
		/* first time accessing this device, let's create it */
        tiny = kmalloc (sizeof (*tiny), GFP_KERNEL);
        if (!tiny) {
            MOD_DEC_USE_COUNT;
            return -ENOMEM;
        }


        tiny->current_i2c_serial_callback_read_entry = 0;
        tiny->current_i2c_serial_callback_write_entry = 0;


        for (i = 0; i < MAX_I2C_SERIAL_CALLBACK_ENTRIES; i ++) {
            i2c_serial_callback_read_tq[min][i].routine = (void (*)(void *))i2c_serial_callback_read;
            i2c_serial_callback_write_tq[min][i].routine = (void (*)(void *))i2c_serial_callback_write;
        }
 
		tiny_table[min] = tiny;
	}

	/* save our structure within the tty structure */
	tty->driver_data = tiny;
	tiny->tty = tty;

    /* FIXME - disable LM intr inthis slot */

#ifdef PIC_MME_READY

    down(&i2c_lock);
    /* prep the PIC */
    if (mme_clearConsoleInterrupt(i2c_base_addr, 1, min) < 0) {
        printk("Clear Console Intr Error\n");
    }
    up(&i2c_lock);

    down(&i2c_lock);
    if (enable_IMxInt_L(i2c_base_addr, 1, min) < 0) {
        printk("Enable IBML Intr Error\n");
    }
    up(&i2c_lock);

    down(&i2c_lock);
    if (mme_enableConsoleInterrupt(i2c_base_addr, 1, min) < 0) {
        printk("Enable Console Intr Error\n");
    }
    up(&i2c_lock);

#endif PIC_MME_READY


	return 0;
}

static void do_close(struct tiny_serial *tiny)
{
    struct tty_struct *tty;

    tty = tiny->tty;

    kfree (tiny);
    tiny_table[minor(tty->device)] = NULL;

    MOD_DEC_USE_COUNT;

}  



static void tiny_close (struct tty_struct *tty, struct file * filp)
{
	struct tiny_serial *tiny = tty->driver_data;
#ifdef PIC_MME_READY
    unsigned char slot = minor(tty->device);
#endif PIC_MME_READY

	if (!tiny) {
		return;
	}

    do_close(tiny);

    /* FIXME - re-enable LM intr for this slot */

#ifdef PIC_MME_READY

    /* disable console intr */
    if (mme_disableConsoleInterrupt(i2c_base_addr, 1, slot) < 0) {
        printk("Disable Console Intr Error\n");
    }

#endif PIC_MME_READY


}



static int tiny_write (struct tty_struct * tty, int from_user, const unsigned char *buf, int count)
{
	struct tiny_serial *tiny = tty->driver_data;
	int retval = -EINVAL;
    int i, j;
    int data;
    unsigned char wbuf[2048];
    unsigned char slot = minor(tty->device);

	if (!tiny) {
		return -ENODEV;
	}

    if (slot == 0) {	/* no support for slot 0 */
        return count;
    }

/* printk("Write %d bytes\n", count); */

    if (from_user) {
        copy_from_user(wbuf, buf, count);
    }
    else {
        for (i = 0; i < count; i ++) {
            wbuf[i] = buf[i];
        }
    }


  for (j = 0; j < count; j += 2) {

    data = 0;

    if (count > (j + 2)) {
       data = 2;
    }
    else {
       data = count - j;
    }

    for (i = 0; i < 2; i ++) {
        data <<= 8;
        data |= wbuf[j + i];
    }

    data <<= 8;
    data |= slot;

    /* data contains: (from MSB) byte 0 = count, byte 1 = buf[0], byte 2 = buf[1], byte 3 = slot */

    i2c_serial_callback_write_tq[slot][tiny->current_i2c_serial_callback_write_entry].data = (void *) data;
    schedule_task(&(i2c_serial_callback_write_tq[slot][tiny->current_i2c_serial_callback_write_entry]));

    tiny->current_i2c_serial_callback_write_entry ++;
    if (tiny->current_i2c_serial_callback_write_entry == MAX_I2C_SERIAL_CALLBACK_ENTRIES) {
        tiny->current_i2c_serial_callback_write_entry = 0;
    }
  }


	retval = count;

}


static void i2c_serial_callback_write(void *arg)
{
    int count;
    int data;
    int numbytes;
    unsigned char slot;

    data = (int) arg;

    slot = data & 0xff;
    numbytes = (data >> 24) & 0xff;

    data <<= 8;

    down (&i2c_lock);


    count = phy_iic_xfer(i2c_base_addr, (mux_select[slot] << 8) | GATEWAY_I2C_CONTROLLER_SLAVE_ADDR, WRITEACCESS, (char *) &data, numbytes, 1);

    if (count != numbytes) {       /* error case */
        printk(KERN_DEBUG "I2C Serial: Write Error\n");
        up (&i2c_lock);
        return;
    }

    up (&i2c_lock);

    return;
}



int cc_i2c_serial_int_handler(int slot)
{
    struct tiny_serial *tiny = tiny_table[slot];

    if (!tiny) {
        return -1;
    }

    /* just to make sure */
    if (slot == 0) {
        return -1;
    }

    i2c_serial_callback_read_tq[slot][tiny->current_i2c_serial_callback_read_entry].data = (void *) slot;

    schedule_task(&(i2c_serial_callback_read_tq[slot][tiny->current_i2c_serial_callback_read_entry]));

    tiny->current_i2c_serial_callback_read_entry ++;
    if (tiny->current_i2c_serial_callback_read_entry == MAX_I2C_SERIAL_CALLBACK_ENTRIES) {
        tiny->current_i2c_serial_callback_read_entry = 0;
    }

    return 0;
}





static void i2c_serial_callback_read(void *arg)
{
    unsigned char buf[MAX_READ_SIZE];
    int slot;
    struct tiny_serial *tiny;
    struct tty_struct *tty;
    int count;
    int nread;
    unsigned char c;
    int i;
#ifdef PIC_MME_READY
    int isConsoleInterrupt = 0;
#endif PIC_MME_READY

    slot = (int) arg;


    tiny = tiny_table[slot];

    if (!tiny) {
        return;
    }


    tty = tiny->tty;

    if (tty->flip.count >= TTY_FLIPBUF_SIZE) {
        tty_flip_buffer_push(tty);
    }

#ifdef PIC_MME_READY

    down (&i2c_lock);

    if (mme_checkConsoleInterrupt(i2c_base_addr, 1, slot, &isConsoleInterrupt) < 0) {
        printk("Check Console Intr Error\n");
    }

    if (isConsoleInterrupt != 0) {	/* not console, so let the system handle it */
        printk("Not for console\n");
        wake_up_interruptible(&sysevent_waitq);
        up(&i2c_lock);
        return;
    }

    up(&i2c_lock);

    /* if here, it is console intr. Clear it */
    down (&i2c_lock);

    if (mme_clearConsoleInterrupt(i2c_base_addr, 1, slot) < 0) {
        printk("Clear Console Intr Error\n");
    }

    up(&i2c_lock);

#endif PIC_MME_READY



    down (&i2c_lock);

    /* count = simple_iic_xfer(i2c_base_addr, GATEWAY_I2C_CONTROLLER_SLAVE_ADDR, READACCESS, buf, 4); */

    /* FIXME - when the PIC is ready with interrupt registers, we need to clear the intr register inthe PIC here with an i2c transaction */
 
    nread = phy_iic_xfer(i2c_base_addr, (mux_select[slot] << 8) | GATEWAY_I2C_CONTROLLER_SLAVE_ADDR, READACCESS, buf, 3, 1);

    up(&i2c_lock);

    if (nread != 3) {		/* error case */
        printk(KERN_DEBUG "I2C Serial: Read Error\n");
        goto i2c_serial_callback_read_exit;
    }

    count = (buf[1] << 8) | buf[0];

    if (count == 0) {	/* bogus reading. Maybe the other side is not ready */
        return;
    }


    c = buf[2];
    tty_insert_flip_char(tty, c, 0);

    count --;       /* we just read 1 byte */

    if (count == 0) {
        goto i2c_serial_callback_read_exit;
    }

    if (count > MAX_READ_SIZE) {
        printk(KERN_DEBUG "I2C Serial:Read buffer overflow\n");
        count = MAX_READ_SIZE;
    }

    /* if here, there is more to read */
    down (&i2c_lock);
    nread = phy_iic_xfer(i2c_base_addr, (mux_select[slot] << 8) | GATEWAY_I2C_CONTROLLER_SLAVE_ADDR, READACCESS, buf, count, 1);
    up(&i2c_lock);

    if (nread != count) {
        printk(KERN_DEBUG "I2C Serial:Read Error\n");
        goto i2c_serial_callback_read_exit;
    }
    

    /* this doesn't actually push the data through unless tty->low_latency is set */
    for (i = 0; i < count; i ++) { 
        tty_insert_flip_char(tty, buf[i], 0);
    }


i2c_serial_callback_read_exit:

#if 0
    down(&i2c_lock);

    /* signal read completion */ 
    phy_iic_xfer(i2c_base_addr, (mux_select[slot] << 8) | GATEWAY_I2C_CONTROLLER_SLAVE_ADDR, WRITEACCESS, buf, 3, 1);

    up (&i2c_lock);

#endif

    tty_flip_buffer_push(tty);
    tty_schedule_flip (tty);

    return;
}

static void tiny_flush_chars(struct tty_struct *tty)
{
    /* printk("In flush_chars funtion\n"); */
}




static int tiny_write_room (struct tty_struct *tty) 
{
	struct tiny_serial *tiny = tty->driver_data;
	int room;

	if (!tiny) {
		return -ENODEV;
	}

	/* calculate how much room is left in the device */
	room = 3;

	return room;
}

static int tiny_chars_in_buffer(struct tty_struct *tty)
{
    /* printk("In chars_in_buffer\n"); */
    return 0;
}



static int tiny_ioctl(struct tty_struct *tty, struct file *file, unsigned int cmd, unsigned long arg)
{
    /* printk("In ioctl, cmd = %d\n", cmd); */
    return -ENOIOCTLCMD;
}


static void tiny_set_termios(struct tty_struct *tty, struct termios *old)
{
    /* printk("In set_termios\n"); */
}

static void tiny_throttle(struct tty_struct *tty)
{
    /* printk("In throttle\n"); */
}

static void tiny_unthrottle(struct tty_struct *tty)
{
    /* printk("In unthrottle\n"); */
}

static void tiny_stop(struct tty_struct *tty)
{
    /* printk("In stop\n"); */
}

static void tiny_start(struct tty_struct *tty)
{
    /* printk("In start\n"); */
}

static void tiny_hangup(struct tty_struct *tty)
{
    /* printk("In hangup\n"); */
}

static void tiny_break_ctl(struct tty_struct *tty, int state)
{
    /* printk("In break_ctl\n"); */
}

static void tiny_flush_buffer(struct tty_struct *tty)
{
    /* printk("In flush_buffer\n");  */
}

static void tiny_set_ldisc(struct tty_struct *tty)
{
    /* printk("In set_ldisc\n"); */
}

static void tiny_wait_until_sent(struct tty_struct *tty, int timeout)
{
    /* printk("In wait_until_sent\n"); */
}

static void tiny_send_xchar(struct tty_struct *tty, char ch)
{
    /* printk("In send_xchar\n"); */
}








static struct tty_driver tiny_tty_driver = {
	magic:			TTY_DRIVER_MAGIC,
	driver_name:	"i2ctty",
	name:			"i2ctty",
	major:			TINY_TTY_MAJOR,
    minor_start:	0,
	num:			TINY_TTY_MINORS,
	type:			TTY_DRIVER_TYPE_SERIAL,
	subtype:		SERIAL_TYPE_NORMAL,
	flags:			TTY_DRIVER_REAL_RAW,
	
	refcount:		&tiny_refcount,
	table:			tiny_tty,
	termios:		tiny_termios,
	termios_locked:		tiny_termios_locked,

	open:			tiny_open,
	close:			tiny_close,
	write:			tiny_write,
    flush_chars:	tiny_flush_chars,
	write_room:		tiny_write_room,
    chars_in_buffer:	tiny_chars_in_buffer,
    ioctl:			tiny_ioctl,
    set_termios:	tiny_set_termios,
    throttle:		tiny_throttle,
    unthrottle:		tiny_unthrottle,
    stop:			tiny_stop,
    start:			tiny_start,
    hangup:			tiny_hangup,
    break_ctl:		tiny_break_ctl,
    flush_buffer:	tiny_flush_buffer,
    set_ldisc:		tiny_set_ldisc,
    wait_until_sent:	tiny_wait_until_sent,
    send_xchar:		tiny_send_xchar,
};

int __init i2c_serial_init(void)
{
	/* register the tty driver */
	tiny_tty_driver.init_termios          = tty_std_termios;

	if (tty_register_driver (&tiny_tty_driver)) {
		printk (KERN_ERR "failed to register I2C tty driver\n");
		return -1;
	}

	printk (KERN_INFO DRIVER_DESC " " DRIVER_VERSION "\n");

    pic_base_addr = ioremap_nocache(PIC_PHY_ADDR, 1);
    i2c_base_addr = ioremap_nocache(I2C_CONTROLLER1_PHY_ADDR, 0x12);

	return 0;
}

void __exit i2c_serial_exit(void)
{
	struct tiny_serial *tiny;
	int i;

	tty_unregister_driver(&tiny_tty_driver);
    iounmap(pic_base_addr);
    iounmap(i2c_base_addr);

	/* shut down all of the timers and free the memory */
	for (i = 0; i < TINY_TTY_MINORS; ++i) {
		tiny = tiny_table[i];
		if (tiny) {
		    do_close (tiny);
		}
	}
}


int i2c_serial_slave_write_handler(unsigned char slave_status)
{
    return -1;
}


int i2c_serial_slave_read_handler(unsigned char slave_status)
{
    return -1;
}




