/***************************************************************************
*
*   Module:             diags
*
*   Filename:           sysevent.h
*
*   Author:             Brian Tran
*
*   Created:            09/03/2002
*
*   Description:
*
*   For system event interrupt
*
****************************************************************************
*                 Source Control System Information
*
*    $Id: cc_sysevent.h,v 1.2 2003/12/15 18:47:23 brian Exp $
*
****************************************************************************
*                       Modification History
*
*    $Log: cc_sysevent.h,v $
*    Revision 1.2  2003/12/15 18:47:23  brian
*    Bugzilla number:
*    Added debug messages
*
*    Revision 1.1  2003/07/25 00:26:07  brian
*    Bugzilla number:
*    I2C console support for IGR_CC
*
*    Revision 1.5  2003/05/08 21:00:27  brian
*    Bugzilla number:
*    Added more interrupt types
*
*    Revision 1.4  2003/05/05 23:46:59  scott
*    merge from 1.1.2 branch
*
*    Revision 1.3.2.1  2003/05/03 06:20:40  brian
*    Bugzilla number:
*    Add async event support for switch cards
*
*    Revision 1.3  2003/04/30 19:00:41  brian
*    Bugzilla number:
*    Added support for async event handling in IGR CC
*
*    Revision 1.2  2003/03/11 02:24:05  brian
*    Check power supply not present
*
*    Revision 1.1  2003/03/06 05:49:46  brian
*    For sysevent interrupts
*
*
***************************************************************************
*                       (c) COPYRIGHT 2000, 2001, 2002
*                           TopSpin Communications, Inc.
*                           Palo Alto, CA
**************************************************************************/


#ifndef _SYSEVENT_H
#define _SYSEVENT_H


#define SYSEVENT_MAJOR  250
#define SYSEVENT_MINOR  0

#define FPGA_REGADDR_BASE            0x1f0100000



#define FPGA_STATUS1_REGADDR         0x3
#define FPGA_STATUS2_REGADDR         0x5
#define FPGA_STATUS3_REGADDR         0x6
#define FPGA_STATUS4_REGADDR         0x7

#define FPGA_INT1_REGADDR          0x8
#define FPGA_INT2_REGADDR          0x9

#define FPGA_MASK1_REGADDR          0xa
#define FPGA_MASK2_REGADDR          0xb

#define FPGA_CHANGE1_REGADDR          0xc
#define FPGA_CHANGE2_REGADDR          0xd
#define FPGA_CHANGE3_REGADDR          0xe

#define FPGA_OTHER_CC_PRESENT_MASK      0x04
#define FPGA_PS2_AC_FAIL_MASK           0x20
#define FPGA_PS1_AC_FAIL_MASK           0x08
#define FPGA_PS2_PRESENT_MASK           0x02
#define FPGA_PS1_PRESENT_MASK           0x01
#define FPGA_FAN2_PRESENT_MASK          0x02
#define FPGA_FAN1_PRESENT_MASK          0x01




#define FPGA_PS_AC_FAIL_MASK		0x80		/* bit 0 */
#define FPGA_PS_FAIL_MASK			0x40		/* bit 1 */
#define FPGA_PS_FAN_FAIL_MASK		0x10		/* bit 3 */
#define FPGA_PS_TYPE1_PRESENT_MASK	0x08		/* bit 4 */
#define FPGA_PS_TYPE2_PRESENT_MASK	0x04		/* bit 5 */

#define FPGA_MEZZANINE_PRESENT_MASK		0x02		/* bit 6 */

#define FPGA_FAN_PRESENT_MASK		0x02		/* bit 6 */
#define FPGA_FAN_FAIL_MASK			0x01		/* bit 7 */

#define GPIO_REGADDR_BASE            0x140000000
#define GPIO_INPUT_REGADDR           0x71c




#define LM_IRQ_NUM        26
#define FPGA_IRQ_NUM        27



#define FPGA_INT_CODE		0x1
#define PS_FAIL_INT_CODE	0x2
#define IBML_INT_CODE		0x3
#define LM_INT_CODE			0x4
#define EXT_INT_CODE		0x5
#define SWITCH_INT_CODE     0x6
#define SLAVE_CC_INT_CODE   0x7
#define FAN_INT_CODE        0x8

#define FPGA_INT_EN       0x11
#define PS_FAIL_INT_EN    0x12
#define IBML_INT_EN       0x13
#define LM_INT_EN         0x14

#define LOOP_ENABLE		  0x15	

#define I2C_DEBUG_CODE		0x10000

#define FPGA_INT_DIS       0x21
#define PS_FAIL_INT_DIS    0x22
#define IBML_INT_DIS       0x23
#define LM_INT_DIS         0x24

#define LOOP_DISABLE		0x25


#endif _SYSEVENT_H

