/* 
 * arch/ppc/platforms/ds1672_rtc.c
 * 
 * Routine to init, read/write to ds1672 rtc 
 */

#include <linux/config.h>
#include <linux/sched.h>
#include <linux/stddef.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/errno.h>
#include <linux/reboot.h>
#include <linux/pci.h>
#include <linux/kdev_t.h>
#include <linux/types.h>
#include <linux/major.h>
#include <linux/blk.h>
#include <linux/console.h>
#include <linux/delay.h>
#include <linux/ide.h>
#include <linux/irq.h>
#include <linux/seq_file.h>
#include <linux/serial.h>
#include <linux/i2c.h>

#include <asm/system.h>
#include <asm/pgtable.h>
#include <asm/page.h>
#include <asm/dma.h>
#include <asm/io.h>
#include <asm/machdep.h>
#include <asm/pci-bridge.h>
#include <asm/time.h>
#ifdef CONFIG_PPC_RTC
#include <asm/todc.h>
#endif /* CONFIG_PPC_RTC */
#include <asm/bootinfo.h>
#include <asm/ppc4xx_pic.h>

#include "ds1672_rtc.h"

#ifdef CONFIG_TS_270_SC
#include "270sc_sysevent.h"
#include <i2c-errcode.h>
#include <fpga.h>
#endif /* CONFIG_TS_270_SC */


/* global */
#ifdef CONFIG_TS_270_SC
unsigned char ts270_my_slot_id;
int ts270_mgmtmux_override;
 
#endif /* CONFIG_TS_270_SC */

extern struct semaphore i2c_lock;

#define DEB(x) if (global_i2c_debug>=1) x
#define DEB2(x) if (global_i2c_debug>=2) x
#define DEB3(x) if (global_i2c_debug>=3) x  /* print several statistical values */

extern int global_i2c_debug;

#ifdef CONFIG_TS_270_SC
int seti2croute(void *vaddr, unsigned char muxselect, int (*iic_xfer)(void *, unsigned short, int, char *, int, int), int controller, unsigned char *mgmt_mux_addr, unsigned char auxselect);
void unseti2croute(void *vaddr, unsigned char muxselect, int (*iic_xfer)(void *, unsigned short, int, char *, int, int), int controller, unsigned char mgmt_mux_addr, unsigned char auxselect);
void * ts270_fpga_base_addr;
#endif /* CONFIG_TS_270_SC */

void ds1672_set_rtc_time_callback(void *arg);

static struct tq_struct rtc_callback_tq = {
    routine: ds1672_set_rtc_time_callback,
};


/* this is a mini i2c driver to read/write rtc.
 * the main kernel i2c driver is not yet initialized at this time.
 */


/* iic_init
 * -------
 * Init the I2C controller. No need to call if we can assune the ppcboot has already init the I2C controller.
 */
#if 0
static int iic_init(void *vaddr)
{

    /* soft reset the controller */
    writeb(0x01, (void *)((int)vaddr + I2C_CONTROLLER_EXT_CONTROL));
    udelay(500);
    writeb(0x0, (void *)((int)vaddr + I2C_CONTROLLER_EXT_CONTROL));
    udelay(500);

    /* Clear master low master address */
    writeb(0x0, (void *)((int)vaddr + I2C_CONTROLLER_LO_MASTER_ADDR));

    /* Clear high master address */
    writeb(0x0, (void *)((int)vaddr + I2C_CONTROLLER_HI_MASTER_ADDR));

    /* Clear low slave address */
    writeb(0x0, (void *)((int)vaddr + I2C_CONTROLLER_LO_SLAVE_ADDR));

    /* Clear high slave address */
    writeb(0x0, (void *)((int)vaddr + I2C_CONTROLLER_HI_SLAVE_ADDR));

    /* Clear status */
    writeb(0x0a, (void *)((int)vaddr + I2C_CONTROLLER_STATUS));

    /* Clear extended status */
    writeb(0x8f, (void *)((int)vaddr + I2C_CONTROLLER_EXT_STATUS));

    /* Set clock division */
    writeb(0x04, (void *)((int)vaddr + I2C_CONTROLLER_CLOCK_DIV));

    /* Disable interrupts on Requested Master Transfer Complete */
    writeb(0x0, (void *)((int)vaddr + I2C_CONTROLLER_INT_MASK));

    /* Clear transfer count */
    writeb(0x0, (void *)((int)vaddr + I2C_CONTROLLER_TRANSFER_COUNT));

    /* Clear extended control and status */
    writeb(0xf0, (void *)((int)vaddr + I2C_CONTROLLER_EXT_CONTROL));

    /* Set mode control (flush master data buf, enable hold SCL, exit */
    /* unknown state.                                                 */
    writeb(0xce, (void *)((int)vaddr + I2C_CONTROLLER_MODE_CONTROL));

    /* Clear control register */
    writeb(0x0, (void *)((int)vaddr + I2C_CONTROLLER_CONTROL));

    return 0;
}
#endif


static int check_i2c_status(void *vaddr)
{
    char status;

    status = readb((void *)((int)vaddr + I2C_CONTROLLER_STATUS));

    /* check the error bit */
    if ((status & 0x4) == 0) {
        return 0;
    }

    DEB(printk("Polled I2C: Error status = %x\n", status));
    return 1;
}


/* wait_til_done
 * -------------
 * Poll the controller until done.
 */
static int wait_til_done(void *vaddr)
{
    int count;
    volatile char status;

    count = 0;
    while (1) {
        status = readb((void *)((int)vaddr + I2C_CONTROLLER_STATUS));

        if ((status & 0x1) == 0) {
            break;
        }

        count ++;

        if (count > MAX_I2C_WAIT) {
            DEB(printk("Polled I2C: Time out waiting for Done, status = %x\n", status));
            return 1;
        }
    }

    return 0;
}

/* iic_readbytes
 * -------------
 * Read from i2c controller. Maximum 4 bytes.
 */
static int iic_readbytes(void *vaddr, char *buf, int count)
{
    int i;
    unsigned char data;

    if ((count == 0) || (count > 4)) {
        return 0;
    }

    data = 0x03 | ((count - 1) << 4);
    writeb(data, (void *)((int)vaddr + I2C_CONTROLLER_CONTROL));

    if (wait_til_done(vaddr) != 0) {
        return 0;
    }

    if (check_i2c_status(vaddr) != 0) {
        return 0;
    }

    for (i = 0; i < count; i++) {
        buf[i] = readb((void *)((int)vaddr + I2C_CONTROLLER_MASTER_DBUF));
    }


    return count;
}

static int iic_sendbytes(void *vaddr, char *buf, int count)
{
    int i;
    char data;

    if ((count == 0) || (count > 4)) {
        return 0;
    }

    for (i = 0; i < count; i++) {
        writeb(buf[i], (void *)((int)vaddr + I2C_CONTROLLER_MASTER_DBUF));
    }

    data = 0x01 | ((count - 1) << 4);

    writeb(data, (void *)((int)vaddr + I2C_CONTROLLER_CONTROL));


    if (wait_til_done(vaddr) != 0) {
        return 0;
    }

    if (check_i2c_status(vaddr) != 0) {
        return 0;
    }


    return count;
}

static int iic_xfer(void *vaddr, unsigned short dev_addr, int access, char *buf, int count, int controller)
{
    char data;

    /* clear status */
    data = 0xa;
    writeb(data, (void *)((int)vaddr + I2C_CONTROLLER_STATUS));

    /* wait for any pending transfer to complete */
    if (wait_til_done(vaddr) != 0) {
        return 0;
    }

    /* flush the buffer */
    data = readb((void *)((int)vaddr + I2C_CONTROLLER_MODE_CONTROL));
    data |= 0x40;
    writeb(data, (void *)((int)vaddr + I2C_CONTROLLER_MODE_CONTROL));

    /* load address */
    data = 0;
    writeb(data, (void *)((int)vaddr + I2C_CONTROLLER_HI_MASTER_ADDR));

    if (access == READACCESS) {
        data = (dev_addr << 1) | 0x1;
    }
    else {
        data = (dev_addr << 1) & 0xfe;
    }

    /* ignore the MSB, which is used for muxselect */
    data = data & 0xff;
    writeb(data, (void *)((int)vaddr + I2C_CONTROLLER_LO_MASTER_ADDR));

    /* check if bus busy */
    data = readb((void *)((int)vaddr + I2C_CONTROLLER_EXT_STATUS));

    /* Mask off the irrelevent bits */
    data = data & 0x70;
    if (data != 0x40) {
        if (data == 0x50) {
            DEB(printk("Polled I2C: Bus busy, ext status = %x\n", data));
            return 0;
        }
        else if ((data == 0x0) || (data == 0x70)) {
            DEB(printk("Polled I2C: Bus bad state, ext status = %x\n", data));
            return 0;
        }
    }

    if (access == READACCESS) {
        return iic_readbytes(vaddr, buf, count);
    }
    else {
        return  iic_sendbytes(vaddr, buf, count);
    }
}


#ifdef CONFIG_TS_270_SC

static int seti2croute_retry(void *vaddr, unsigned char muxselect, int (*iic_xfer)(void *, unsigned short, int, char *, int, int), int controller, unsigned char *mgmt_mux_addr, unsigned char auxselect, int *request_access)
{
    int returncode;
    volatile unsigned char data;
    int count = 0;

    *request_access = 0;
    returncode = seti2croute(vaddr, muxselect, iic_xfer, controller, mgmt_mux_addr, 0);

    if (returncode == IIC_ERR_PIC_UPDATE) {
        /* request access */
        writeb(0x80, (void *) ((int) ts270_fpga_base_addr + SC_270_FPGA_SYSERR_INTPIN_REG_OFFSET));
        *request_access = 1;
        data = readb((void *) ((int) ts270_fpga_base_addr + SC_270_FPGA_SYSERR_INTPIN_REG_OFFSET));
        count = 0;
#ifdef TEST_INTR
        while ((data & 0x01) != 0) {
#else
        while ((data & 0x40) != 0) {
#endif
            udelay(50000);
            count ++;
            if (count > 20) {
                break;
            }
            data = readb((void *) ((int) ts270_fpga_base_addr + SC_270_FPGA_SYSERR_INTPIN_REG_OFFSET));
        }
        /* retry */
        returncode = seti2croute(vaddr, muxselect, iic_xfer, controller, mgmt_mux_addr, 0);
    }

    return returncode;
}

static void unseti2croute_retry(void *vaddr, unsigned char muxselect, int (*iic_xfer)(void *, unsigned short, int, char *, int, int), int controller, unsigned char mgmt_mux_addr, unsigned char auxselect, int request_access)
{
    unseti2croute(vaddr, muxselect, iic_xfer, controller, mgmt_mux_addr, auxselect);

    if (request_access == 1) {
        writeb(0x0, (void *) ((int) ts270_fpga_base_addr + SC_270_FPGA_SYSERR_INTPIN_REG_OFFSET));
        udelay(100000);
    }

}



#endif /* CONFIG_TS_270_SC */





/* ds1672_time_init
 * ----------------
 * Make sure the RTC is in couting mode.
 * Return zero.
 */
long ds1672_time_init(void)
{
    char buf[2];    /* 1 data byte + 1 addr byte */
    void *vaddr;
    int controller = 0;

#ifdef CONFIG_TS_270_SC
    unsigned char data;	
    /* unsigned char ts270_my_slot_id; */
    unsigned char mgmt_mux_addr = 0;
    unsigned char muxselect;
    unsigned char next_muxselect;
    unsigned char othercore_present;
    int request_access = 0;

    ts270_mgmtmux_override = 0;

    ts270_fpga_base_addr = ioremap_nocache(FPGA_REGADDR_BASE, 0x200);
    ts270_my_slot_id = SC_270_FPGA_FABRIC_SLOT_ID_MASK & readb((void *) ((int) ts270_fpga_base_addr + SC_270_FPGA_MOD_CHASID_REG_OFFSET)); 
    data = readb((void *) ((int) ts270_fpga_base_addr + SC_270_FPGA_MOD_PRES_REG2_OFFSET));
    othercore_present = readb((void *) ((int) ts270_fpga_base_addr + SC_270_FPGA_MOD_PRES_REG0_OFFSET));
    othercore_present &= 0x08;

    if ((ts270_my_slot_id != SC_270_FPGA_CORE1_SLOT_NUM) && (ts270_my_slot_id != SC_270_FPGA_CORE2_SLOT_NUM)) {
        return 0;
    }

    /* if no mgmt cards present, return */
    if ((data & 0x0c) == 0) {
        return 0;
    }

    /* both mgmt cards present. If no other core, set RTC to both. Otherwise, only set my mgmt card */
    if ((data & 0x0c) == 0x0c) {	/* both mgmt cards present */
        if (othercore_present != 0) {	/* other core present */
            if (ts270_my_slot_id == SC_270_FPGA_CORE1_SLOT_NUM) {   /* slot 11 */
                muxselect = 0x60;
            }
            else {
                muxselect = 0x62;
            }
            next_muxselect = 0x0;        
        }
        else {	/* other core not present, set both cards */
            muxselect = 0x60;
            next_muxselect = 0x62;
        }
    }


    if ((data & 0x0c) == 0x08) {   /* only mgmtA present */
        muxselect = 0x60;
        next_muxselect = 0x0;
    }

    if ((data & 0x0c) == 0x04) {   /* only mgmtB present */
        muxselect = 0x62;
        next_muxselect = 0x0;
    }

#if 0
    /* before accessing the mgmt card, assert error out so that the other core cannot access this mgmt card */
    writeb(0x40, (void *) ((int) ts270_fpga_base_addr + SC_270_FPGA_SYSERR_INTPIN_REG1_OFFSET));
    /* delay half a second */
    udelay(500000);
#endif


#endif

    buf[0] = 4;     /* offset 4 for control */
    buf[1] = 0;     /* enable couting */

    vaddr = ioremap_nocache(I2C_CONTROLLER0_BASE, 0x12);


    /* no need to check return code. The kernel code in kernel/time.c will detect if the RTC is stuck */
    DEB(printk("Polled I2C: Acquiring I2C Lock\n"));
    down(&i2c_lock);
    DEB(printk("Polled I2C: Got I2C Lock\n"));

#ifdef CONFIG_TS_270_SC
    if (seti2croute_retry(vaddr, muxselect, iic_xfer, controller, &mgmt_mux_addr, 0, &request_access) == 0) {
        iic_xfer(vaddr, RTC_I2C_ADDR, WRITEACCESS, buf, 2, controller);
    }
        
    unseti2croute_retry(vaddr, 0xff, iic_xfer, controller, mgmt_mux_addr, 0, request_access);

#else

    iic_xfer(vaddr, RTC_I2C_ADDR, WRITEACCESS, buf, 2, controller);

#endif /* CONFIG_TS_270_SC */

    up(&i2c_lock);
    DEB(printk("Polled I2C: Released I2C Lock\n"));

#ifdef CONFIG_TS_270_SC
    if (next_muxselect != 0) {
        down(&i2c_lock);
        if (seti2croute_retry(vaddr, next_muxselect, iic_xfer, controller, &mgmt_mux_addr, 0, &request_access) == 0) {
            iic_xfer(vaddr, RTC_I2C_ADDR, WRITEACCESS, buf, 2, controller);
        }
        unseti2croute_retry(vaddr, 0xff, iic_xfer, controller, mgmt_mux_addr, 0, request_access);
        up(&i2c_lock);
    }
#endif /* CONFIG_TS_270_SC */

    iounmap(vaddr);

    return 0;
}


unsigned long ds1672_get_rtc_time(void)
{
    char buf[4];
    void *vaddr;
    int count;
    int i;
    unsigned long rtc_count;
    int controller = 0;

#ifdef CONFIG_TS_270_SC
    unsigned char data;
    /* unsigned char ts270_my_slot_id; */
    unsigned char muxselect = 0;
    unsigned char mgmt_mux_addr = 0;
    int request_access;

    /* ts270_my_slot_id = SC_270_FPGA_FABRIC_SLOT_ID_MASK & readb((void *) ((int) ts270_fpga_base_addr + SC_270_FPGA_MOD_CHASID_REG_OFFSET)); */
    data = readb((void *) ((int) ts270_fpga_base_addr + SC_270_FPGA_MOD_PRES_REG2_OFFSET));

    if ((ts270_my_slot_id != SC_270_FPGA_CORE1_SLOT_NUM) && (ts270_my_slot_id != SC_270_FPGA_CORE2_SLOT_NUM)) {
        udelay(1000);
        return 0;
    }

    /* if no mgmt cards present, return */
    if ((data & 0x0c) == 0) {
        udelay(1000);
        return 0;
    }

    /* both mgmt cards present, choice is based on my slot */
    if ((data & 0x0c) == 0x0c) {
        if (ts270_my_slot_id == SC_270_FPGA_CORE1_SLOT_NUM) {	/* slot 11 */
            muxselect = 0x60;
        }
        else {
            muxselect = 0x62;
        } 
    }

    if ((data & 0x0c) == 0x08) {   /* only mgmtA present */
        muxselect = 0x60;
    }

    if ((data & 0x0c) == 0x04) {   /* only mgmtB present */
        muxselect = 0x62;
    }


#endif /* CONFIG_TS_270_SC */

    buf[0] = buf[1] = buf[2] = buf[3] = 0;

    vaddr = ioremap_nocache(I2C_CONTROLLER0_BASE, 0x12);


    DEB(printk("Polled I2C: Acquiring I2C Lock\n"));
    down(&i2c_lock);
    DEB(printk("Polled I2C: Got I2C Lock\n"));

#ifdef CONFIG_TS_270_SC
    if (seti2croute_retry(vaddr, muxselect, iic_xfer, controller, &mgmt_mux_addr, 0, &request_access) != 0) {
        goto ds1672_get_rtc_time_error;
    }

#endif /* CONFIG_TS_270_SC */

    /* write transaction: controller 0, address 0x68, 1 byte data = 0x0 */
    count = iic_xfer(vaddr, RTC_I2C_ADDR, WRITEACCESS, buf, 1, controller);
    if (count != 1) {
        goto ds1672_get_rtc_time_error;
    }

    /* read transaction: controller 0, address 0x68, 4 byte read */
    count = iic_xfer(vaddr, RTC_I2C_ADDR, READACCESS, buf, 4, controller);
    if (count != 4) {
        goto ds1672_get_rtc_time_error;
    }

#ifdef CONFIG_TS_270_SC
    unseti2croute_retry(vaddr, 0xff, iic_xfer, controller, mgmt_mux_addr, 0, request_access);
#endif /* CONFIG_TS_270_SC */


    up(&i2c_lock);
    DEB(printk("Polled I2C: Released I2C Lock\n"));

    iounmap(vaddr);

    /* swap bytes */
    rtc_count = 0;
    for (i = 3; i >= 0; i --) {
        rtc_count <<= 8;
        rtc_count |= buf[i] & 0xff;
    }


    return rtc_count;

ds1672_get_rtc_time_error:


#ifdef CONFIG_TS_270_SC
    unseti2croute_retry(vaddr, 0xff, iic_xfer, controller, mgmt_mux_addr, 0, request_access);
#endif /* CONFIG_TS_270_SC */


    up(&i2c_lock);
    DEB(printk("Polled I2C: Released I2C Lock\n"));

    iounmap(vaddr);

    return 0;

}


int ds1672_set_rtc_time(unsigned long nowtime)
{
    rtc_callback_tq.data = (void *) nowtime;
    schedule_task(&rtc_callback_tq);

    return 0;
}



void ds1672_set_rtc_time_callback(void *arg)
{
    char buf1[3];   /* 2 data byte + 1 addr byte */
    char buf2[3];   /* 2 data byte + 1 addr byte */
    void *vaddr;
    int i;
    unsigned long nowtime;
    int controller = 0;
    int returncode = 0;

#ifdef CONFIG_TS_270_SC
    unsigned char data;
    /* unsigned char ts270_my_slot_id; */
    unsigned char muxselect;
    unsigned char next_muxselect;
    unsigned char mgmt_mux_addr = 0;
    unsigned char othercore_present;
    unsigned char am_i_master;
    int request_access;

    am_i_master = SC_270_FPGA_MASTER_A_DETECT_MASK & readb((void *) ((int) ts270_fpga_base_addr + SC_270_FPGA_MOD_CHASID_REG_OFFSET)); 

    if (am_i_master == 0) {
        return;
    }

    data = readb((void *) ((int) ts270_fpga_base_addr + SC_270_FPGA_MOD_PRES_REG2_OFFSET));
    othercore_present = readb((void *) ((int) ts270_fpga_base_addr + SC_270_FPGA_MOD_PRES_REG0_OFFSET));
    othercore_present &= 0x08;

    if ((ts270_my_slot_id != SC_270_FPGA_CORE1_SLOT_NUM) && (ts270_my_slot_id != SC_270_FPGA_CORE2_SLOT_NUM)) {
        return;
    }

    /* if no mgmt cards present, return */
    if ((data & 0x0c) == 0) {
        return;
    }

    /* both mgmt cards present. Set both cards */
    if ((data & 0x0c) == 0x0c) {    /* both mgmt cards present */
        muxselect = 0x60;
        next_muxselect = 0x62;
    }


    if ((data & 0x0c) == 0x08) {   /* only mgmtA present */
        muxselect = 0x60;
        next_muxselect = 0x0;
    }

    if ((data & 0x0c) == 0x04) {   /* only mgmtB present */
        muxselect = 0x62;
        next_muxselect = 0x0;
    }


#endif


    nowtime = (unsigned long) arg;

    buf1[0] = 0;        /* offset 0 */

    for (i = 1; i <= 2; i++) {
        buf1[i] = nowtime & 0xff;
        nowtime = nowtime >> 8;
    }

    buf2[0] = 2;        /* offset 2 */

    for (i = 1; i <= 2; i++) {
        buf2[i] = nowtime & 0xff;
        nowtime = nowtime >> 8;
    }


    vaddr = ioremap_nocache(I2C_CONTROLLER0_BASE, 0x12);

    DEB(printk("Polled I2C: Acquiring I2C Lock\n"));
    down(&i2c_lock);
    DEB(printk("Polled I2C: Got I2C Lock\n"));


#ifdef CONFIG_TS_270_SC
    returncode = seti2croute_retry(vaddr, muxselect, iic_xfer, controller, &mgmt_mux_addr, 0, &request_access);
#endif /* CONFIG_TS_270_SC */

    if (returncode == 0) {
        iic_xfer(vaddr, RTC_I2C_ADDR, WRITEACCESS, buf1, 3, controller);
        iic_xfer(vaddr, RTC_I2C_ADDR, WRITEACCESS, buf2, 3, controller);
    }

#ifdef CONFIG_TS_270_SC
    unseti2croute_retry(vaddr, 0xff, iic_xfer, controller, mgmt_mux_addr, 0, request_access);
#endif /* CONFIG_TS_270_SC */


   up(&i2c_lock);
   DEB(printk("Polled I2C: Released I2C Lock\n"));


#ifdef CONFIG_TS_270_SC
    if (next_muxselect != 0) {
        down(&i2c_lock);
        if (seti2croute_retry(vaddr, next_muxselect, iic_xfer, controller, &mgmt_mux_addr, 0, &request_access) == 0) {
            iic_xfer(vaddr, RTC_I2C_ADDR, WRITEACCESS, buf1, 3, controller);
            iic_xfer(vaddr, RTC_I2C_ADDR, WRITEACCESS, buf2, 3, controller);
        }
        unseti2croute_retry(vaddr, 0xff, iic_xfer, controller, mgmt_mux_addr, 0, request_access);
        up(&i2c_lock);
    }
#endif /* CONFIG_TS_270_SC */

    iounmap(vaddr);

    return;

}




