/*
*  Copyright (C) 2002 Scott McNutt <smcnutt@artesyncp.com>
*
* See file CREDITS for list of people who contributed to this
* project.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston,
* MA 02111-1307 USA
*/


#include <ppcboot.h>
#include "ts_bld_sc.h"
#include <asm/processor.h>
#include "fpga.h"


long int fixed_sdram( void );
void priority_setup(void);

extern int dontboot;
extern int gpio_set(int, unsigned int, int);

/* local defines */
#define ANAFA_1_RESET_GPIO 2

int board_pre_init (void)
{
    uint reg;
    
    /*--------------------------------------------------------------------
     * GPIO map
     *
     * GPIO0 PCI ANA1 INTR
     * GPIO1 PIC CPU INTR
     * GPIO2 ANAFA2 #1 RESET OUTPUT
     * GPIO3 ENET PHY INTR
     * GPIO4 CPU WATCHDOG TOGGLE OUTPUT
     * GPIO5 FPGA INTR
     * GPIO6 FPGA PROG ENABLE OUT
     * GPIO7 CPU LVPGEN OUT
     * GPIO8 CPU PIC VPP OUTPUT
     * GPIO9 CPU PIC PGM CLK OUTPUT
     * GPIO10 CPU PIC PGM DATA OUTPUT
     * GPIO11 CPU PIC PGM ENABLE OUTPUT
     * GPIO12 FPGA INIT_N
     * GPIO13 NC
     * GPIO14 FPGA DONE 
     * GPIO15 NC
     * GPIO16 NC
     * GPIO17 NC
     *-------------------------------------------------------------------*/

    /* Configure GPIO0-17 as listed above. Configure GPIO18-31 as gpios */
#if 1
	/* 440 docs imply that bits 12 & 14 must be set to 1 to operate as gpio's */
    mtdcr( cpc0_gpio, 0x2bfa0000 );
#else
    mtdcr( cpc0_gpio, 0x2bf00000 );
#endif

    /*--------------------------------------------------------------------
     * Setup the external bus controller/chip selects
     *-------------------------------------------------------------------*/
    mtdcr( ebccfga, xbcfg );
    reg = mfdcr( ebccfgd );
    reg |= 0x04000000;          /* Set ATC */
    reg |= 0x38000000;          /* Set ready timeout to 2K cycles */
    mtdcr( ebccfgd, reg );

    mtebm( ebmctl, 0x84480000 ); /* External Bus Master; bus width 16bits, parity off */

    mtebc( pb0ap, 0x03814480 ); /* DiskOnChip WBN=1 increase WE setup time */
    mtebc( pb0cr, 0xfff18000 ); /* BA=0xfff 128MB R/W 8bit */

    /* Legacy from openbios */
    mtebc( pb1ap, 0x02815480 );	/* NVRAM/RTC */
    mtebc( pb1cr, 0x48018000 ); /* BA=0x480 1MB R/W 8-bit */

    mtebc( pb2ap, 0x00000200 );	/* Xilinx FPGA */
    mtebc( pb2cr, 0xf0118000 ); /* BA=0xf01 1MB R/W 8-bit */
    
    mtebc( pb3ap, 0x00000380 );	/* Anafa2 #1 TH=1 RE=1 SOR=1 */
    mtebc( pb3cr, 0xf023a000 ); /* BA=0xf02 2MB R/W 16-bit */

    /* PIC chip select line */
    mtebc( pb4ap, 0x03810480 ); /* PIC */
    mtebc( pb4cr, 0xf0818000 );

    /* Legacy from openbios */
    mtebc( pb7ap, 0x01015280 );	/* FPGA registers */
    mtebc( pb7cr, 0x48318000 ); /* BA=0x483 1MB R/W 8-bit */

    /*--------------------------------------------------------------------
     * Setup the interrupt controller polarities, triggers, etc.
     *-------------------------------------------------------------------*/
    mtdcr( uic0sr, 0xffffffff );    /* clear all */
    mtdcr( uic0er, 0x00000000 );    /* disable all */
    mtdcr( uic0cr, 0x00000009 );    /* SMI & UIC1 crit are critical */
    mtdcr( uic0pr, 0xfffffe13 );    /* per ref-board manual */
    mtdcr( uic0tr, 0x01c00008 );    /* per ref-board manual */
    mtdcr( uic0vr, 0x00000001 );    /* int31 highest, base=0x000 */
    mtdcr( uic0sr, 0xffffffff );    /* clear all */
    
    mtdcr( uic1sr, 0xffffffff );    /* clear all */
    mtdcr( uic1er, 0x00000000 );    /* disable all */
    mtdcr( uic1cr, 0x00000000 );    /* all non-critical */
    mtdcr( uic1pr, 0xffffe0ff );    /* per ref-board manual */
    mtdcr( uic1tr, 0x00ffc000 );    /* per ref-board manual */
    mtdcr( uic1vr, 0x00000001 );    /* int31 highest, base=0x000 */
    mtdcr( uic1sr, 0xffffffff );    /* clear all */

	*(uint32_t *)GPIO_TCR = 0x20000000L;
    priority_setup();

    return 0;
}



int checkboard (void)
{
    sys_info_t sysinfo;
    get_sys_info(&sysinfo);

    printf("Topspin BladeCenter Switch Card\n");
    printf("\tVCO: %lu MHz\n", sysinfo.freqVCOMhz/1000000);
    printf("\tCPU: %lu MHz\n", sysinfo.freqProcessor/1000000);
    printf("\tPLB: %lu MHz\n", sysinfo.freqPLB/1000000);
    printf("\tOPB: %lu MHz\n", sysinfo.freqOPB/1000000);
    printf("\tEPB: %lu MHz\n", sysinfo.freqEPB/1000000);
    return (0);
}


long int initdram (int board_type)
{
    long    dram_size = 0;
#if defined(CONFIG_SPD_EEPROM)
    dram_size = spd_sdram();
#else
    dram_size = fixed_sdram();
#endif
  return dram_size;
}


#if defined(CFG_DRAM_TEST)
int testdram(void)
{
    uint    *pstart = (uint *)0x00000000;
    uint    *pend   = (uint *)0x08000000;
    uint    *p;

    for( p = pstart; p < pend; p++ )
        *p = 0xaaaaaaaa;

    for( p = pstart; p < pend; p++ )
    {
        if( *p != 0xaaaaaaaa )
        {
            printf("SDRAM test fails at: %08x\n", (uint)p );
            return 1;
        }
    }

    for( p = pstart; p < pend; p++ )
        *p = 0x55555555;

    for( p = pstart; p < pend; p++ )
    {
        if( *p != 0x55555555 )
        {
            printf("SDRAM test fails at: %08x\n", (uint)p );
            return 1;
        }
    }
    return 0;
}
#endif

#if !defined(CONFIG_SPD_EEPROM)
/*************************************************************************
 *  fixed sdram init -- doesn't use serial presence detect.
 *
 *  Assumes:    128 MB, non-ECC, non-registered
 *              PLB @ 133 MHz
 *
 ************************************************************************/
long int fixed_sdram( void )
{
    return( 128 * 1024 * 1024 );    /* 128 MB                           */
}
#endif /* !defined(CONFIG_SPD_EEPROM) */

extern int fpga_program(void);
extern int faninit(void);

void
misc_init_r(bd_t *bd)
{
    unsigned long *revaddr;
    unsigned int rev;


    /* Drive Anafa GPIOs low to put into reset */
#if 0
    printf("Drive Anafa2 #1 reset low...\n");
    if (gpio_set(ANAFA_1_RESET_GPIO, 0, 1) != 0)
        printf("Error setting Anafa2 #1 reset GPIO low\n");
#else
    printf("Drive Anafa2 #1 reset low...with pull down\n");
    if (gpio_set(ANAFA_1_RESET_GPIO, 0, 0) != 0)
        printf("Error setting Anafa2 #1 reset GPIO low\n");
#endif

    fpga_program();

	/* After FPGA load, change bus timing TWT=2 OEN=2 CSN=0 SOR=1 TH=1 */
    mtebc( pb2ap, 0x01020280 );	/* Xilinx FPGA */

    /* at least verify the signature of the FPGA firmware. Just make sure it is not zero */ 
    revaddr = (unsigned long *) (TSBLDSC_FPGA_BASE + FPGA_SIGNATURE_OFFSET);
    rev = in8(revaddr);
    printf ("FPGA Revision Register=0x%x Rev=0x%x\n", revaddr, rev);
    
    if (rev == 0) {
        printf("POST: FPGA bad Rev: 0x%x : FAILED\n", rev);
        /* return -1; */
    } else 
        printf("POST: FPGA Rev:0x%x : PASSED\n", rev);

    bc_pic_wdog_reset();
}

void bc_pic_wdog_reset(void)
{
    unsigned long *picaddr;

    picaddr = (unsigned long *)(0xF0800000);
    out8(picaddr,0x16);	/* 0x16 is 0x68 bit reversed for BC1 */
    printf ("PIC wdog reset\n");
}
void bc_pic_test(void)
{
    unsigned long *picaddr;

    picaddr = (unsigned long *)(0xF0800000);
    out8(picaddr,0x16);	/* 0x16 is 0x68 bit reversed for BC1 */
    printf ("PIC wdog reset\n");
}

