/*
*  Copyright (C) 2002 Scott McNutt <smcnutt@artesyncp.com>
*
* See file CREDITS for list of people who contributed to this
* project.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston,
* MA 02111-1307 USA
*/


#include <ppcboot.h>
#include "ts_igr_cc.h"
#include <asm/processor.h>
#include "fpga.h"
#include "../../include/i2c.h"

long int fixed_sdram( void );
void priority_setup(void);
void blink_front_system_led();

extern int dontboot;
char postresults[200] = "";

#define CC_FPGA_BASE	0xf0100000

int board_pre_init (void)
{
    uint reg;
    
    /* Configure GPIO18-31 as gpios */
    mtdcr( cpc0_gpio, 0 );

    /*--------------------------------------------------------------------
     * Setup the external bus controller/chip selects
     *-------------------------------------------------------------------*/
    mtdcr( ebccfga, xbcfg );
    reg = mfdcr( ebccfgd );
    mtdcr( ebccfgd, reg | 0x04000000 );	/* Set ATC */

    mtebc( pb0ap, 0x03810480 );	/* DiskOnChip */
    mtebc( pb0cr, 0xfff18000 );
    
    /* Legacy from openbios */
    mtebc( pb1ap, 0x02815480 );	/* NVRAM/RTC */
    mtebc( pb1cr, 0x48018000 ); /* BA=0x480 1MB R/W 8-bit */
    
    mtebc( pb2ap, 0x03810480 );	/* Xilinx FPGA */
    mtebc( pb2cr, 0xf0118000 );
    
    /* Legacy from openbios */
    mtebc( pb7ap, 0x01015280 );	/* FPGA registers */
    mtebc( pb7cr, 0x48318000 ); /* BA=0x483 1MB R/W 8-bit */

    /*--------------------------------------------------------------------
     * Setup the interrupt controller polarities, triggers, etc.
     *-------------------------------------------------------------------*/
    mtdcr( uic0sr, 0xffffffff );    /* clear all */
    mtdcr( uic0er, 0x00000000 );    /* disable all */
    mtdcr( uic0cr, 0x00000009 );    /* SMI & UIC1 crit are critical */
    mtdcr( uic0pr, 0xfffffe13 );    /* per ref-board manual */
    mtdcr( uic0tr, 0x01c00008 );    /* per ref-board manual */
    mtdcr( uic0vr, 0x00000001 );    /* int31 highest, base=0x000 */
    mtdcr( uic0sr, 0xffffffff );    /* clear all */
    
    mtdcr( uic1sr, 0xffffffff );    /* clear all */
    mtdcr( uic1er, 0x00000000 );    /* disable all */
    mtdcr( uic1cr, 0x00000000 );    /* all non-critical */
    mtdcr( uic1pr, 0xffffe0ff );    /* per ref-board manual */
    mtdcr( uic1tr, 0x00ffc000 );    /* per ref-board manual */
    mtdcr( uic1vr, 0x00000001 );    /* int31 highest, base=0x000 */
    mtdcr( uic1sr, 0xffffffff );    /* clear all */

    priority_setup();

    return 0;
}



int checkboard (void)
{
    sys_info_t sysinfo;
    get_sys_info(&sysinfo);

    printf("Topspin 360 Controller Card\n");
#ifdef DEBUG
    printf("\tVCO: %lu MHz\n", sysinfo.freqVCOMhz/1000000);
    printf("\tCPU: %lu MHz\n", sysinfo.freqProcessor/1000000);
    printf("\tPLB: %lu MHz\n", sysinfo.freqPLB/1000000);
    printf("\tOPB: %lu MHz\n", sysinfo.freqOPB/1000000);
    printf("\tEPB: %lu MHz\n", sysinfo.freqEPB/1000000);
#endif
    return (0);
}


long int initdram (int board_type)
{
    long    dram_size = 0;
#if defined(CONFIG_SPD_EEPROM)
    dram_size = spd_sdram();
#else
    dram_size = fixed_sdram();
#endif
  return dram_size;
}


#if defined(CFG_DRAM_TEST)
int testdram(void)
{
    uint    *pstart = (uint *)0x00000000;
    uint    *pend   = (uint *)0x08000000;
    uint    *p;

    for( p = pstart; p < pend; p++ )
        *p = 0xaaaaaaaa;

    for( p = pstart; p < pend; p++ )
    {
        if( *p != 0xaaaaaaaa )
        {
            printf("SDRAM test fails at: %08x\n", (uint)p );
            return 1;
        }
    }

    for( p = pstart; p < pend; p++ )
        *p = 0x55555555;

    for( p = pstart; p < pend; p++ )
    {
        if( *p != 0x55555555 )
        {
            printf("SDRAM test fails at: %08x\n", (uint)p );
            return 1;
        }
    }
    return 0;
}
#endif

#if !defined(CONFIG_SPD_EEPROM)
/*************************************************************************
 *  fixed sdram init -- doesn't use serial presence detect.
 *
 *  Assumes:    128 MB, non-ECC, non-registered
 *              PLB @ 133 MHz
 *
 ************************************************************************/
long int fixed_sdram( void )
{
    return( 128 * 1024 * 1024 );    /* 128 MB                           */
}
#endif /* !defined(CONFIG_SPD_EEPROM) */

extern int fpga_program(void);

void
misc_init_r(bd_t *bd)
{
    uchar readback1, readback2, rev;
    int sig;

    if (fpga_program() < 0) {
		/* boot and let FPGAInit fix any problems */
        printf("POST: FPGA load FAILED\n");
        return;
    } else {
        readback1 = in8(CC_FPGA_BASE);
        readback2 = in8(CC_FPGA_BASE+2);
        rev = in8(CC_FPGA_BASE + FPGA_SIGNATURE_OFFSET);
        printf ("POST: FPGA Revision 0x%x=0x%x\n", 
            CC_FPGA_BASE+FPGA_SIGNATURE_OFFSET, rev);
        if ((readback1 == 0xDF) || (readback2 == 0xDF)) {
            dontboot = 1;
            printf("POST: FPGA load succeeded, but read 0x%x=0x%x 0x%x=0x%x FAILED\n",
                CC_FPGA_BASE, readback1, CC_FPGA_BASE+2, readback2);
            return;
        } else 
        printf("POST: FPGA download PASSED\n");
    }
    
    /*
     * FPGA reg 0xf0100004 bit2 (0x20) SLOT NUMBER: 0=in slot1 (left) 1=in slot12 (right)
     * FPGA reg 0xf0100003 bit4 (0x08) drive to say I'm present
     * FPGA reg 0xf0100003 bit5 (0x04) other controller is present
     *
     * If we're in slot1, become the master and turn off GWs
     * If we're in slot12, but the only controller card present, become master and turn off GWs
     */
    {
      volatile unsigned char *fp = (volatile unsigned char *)0xf0100000;
      

      *(fp+3) |= 0x08; /* tell others I'm present */
      udelay(10000); /* wait to make sure other controller announced */
      /* Check for other controller master, but not present - shorted BP */
      if ( ((*(fp+5) & 0x40) == 0x40) && ((*(fp+3) & 0x04) == 0x00) ) {
            dontboot = 1;
			printf("POST: FPGA illegal master present\n");
      }
      /* If we are in the left controller card slot then request the mastership */
      if ( (*(fp+4) & 0x20) == 0 ) {
          *(fp+4) |= 0x80;
          blink_front_system_led();
          printf ("CC: I'm in left controller slot, request master\n");
      /* If we are in the right, but no one in the left, request mastership */
      } else if ( (*(fp+3) & 0x04) == 0) {
          *(fp+4) |= 0x80;
          blink_front_system_led();
          printf ("CC: I'm in right controller slot, no one in left, request master\n");
      } else {
          printf ("CC: I'm a slave\n");
      }
      printf ("CC: Turn off GWs and hold SCs in reset\n");
      *(fp+2) = 0;
      *(fp+1) |= 0x0c;  /* hold the switch cards in reset */
    }	  
}


#define BACKPLANE_PCA_I2C_ADDR      0x67

/* define the address of the PCA registers */
#define PCA_LS0         0x05
#define PCA_LS1         0x06
#define PCA_PSC0        0x01
#define PCA_PSC1        0x03

#define PCA_LED_ON      0x0
#define PCA_LED_OFF     0x1
#define PCA_LED_BLINK0  0x2
#define PCA_LED_BLINK1  0x3

/* define the blink rate for LEDs */
#define TWO_HZ_BLINK_RATE       18      /* 2Hz - period = 0.5 second - PSC = 18 ---- (18 + 1)/38 = 0.5 */
#define ONEFOURTH_HZ_BLINK_RATE 151     /* 1/4Hz - period = 4 second - PSC = 151 ---- (151 + 1)/38 = 4 */

#define FPGA_MUX_SELECT_BACKPLANE       0x7

void blink_front_system_led()
{
    unsigned char data; 
    unsigned char read; 
    unsigned char selector; 
    unsigned char led_num; 
    unsigned char mask; 
    unsigned char muxselect; 
    int i;
    volatile unsigned char *fp = (volatile unsigned char *)0xf0100000;

    /* select bus 0 */
    SetCurrentIicBus(0);

    /* set the mux value to talk to backplane card */
    muxselect = *(fp + 4);
    muxselect &= 0xf0;
    muxselect |= FPGA_MUX_SELECT_BACKPLANE;
    *(fp + 4) = muxselect;


    /* set blink rate to 2 Hz */
    data = TWO_HZ_BLINK_RATE;
    i2c_write(BACKPLANE_PCA_I2C_ADDR, PCA_PSC0, 1, &data, 1);

    /* reset all LEDs to off */
    data = 0x55;
    i2c_write(BACKPLANE_PCA_I2C_ADDR, PCA_LS0, 1, &data, 1);
    i2c_write(BACKPLANE_PCA_I2C_ADDR, PCA_LS1, 1, &data, 1);


    selector = PCA_LS0;
    i2c_read(BACKPLANE_PCA_I2C_ADDR, selector, 1, &read, 1);

    /* to blink fast */
    data = PCA_LED_BLINK0;

    /* for system green LED on the backplane */
    led_num = 3;

    /* determine the right bit position for the right LED */
    mask = 0x3;
    for (i = 0; i < led_num; i++) {
        mask <<= 2;
        data <<= 2;
    }

    read = (read & (~mask)) | data;

    i2c_write(BACKPLANE_PCA_I2C_ADDR, selector, 1, &read, 1);

    return;
}



