/*
*  Copyright (C) 2002 Scott McNutt <smcnutt@artesyncp.com>
*
* See file CREDITS for list of people who contributed to this
* project.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston,
* MA 02111-1307 USA
*/


#include <ppcboot.h>
#include "ts_igr_fc.h"
#include <asm/processor.h>
#include <pci.h>

extern int swrite(volatile unsigned int *addr, unsigned int val);
extern unsigned int sread(volatile unsigned int *addr);

long int fixed_sdram( void );

int longpost = 0;
char postresults[200] = "";

int board_pre_init (void)
{
    uint reg;

    /* Configure pin GPIO6 -> IRQ6, GPIO7-11 as GPIO, GPIO15 is GPIO */
    mtdcr( cpc0_gpio, 0x01f10000 );

    /* GPIO11 as output */
    out32(GPIO_TCR, in32(GPIO_TCR) | 0x00100000);

    /* Disable PIC programming */
    out32(GPIO_OR,  in32(GPIO_OR)  & ~0x00100000);

    /*--------------------------------------------------------------------
     * Setup the external bus controller/chip selects
     *-------------------------------------------------------------------*/
    mtdcr( ebccfga, xbcfg );
    reg = mfdcr( ebccfgd );
    mtdcr( ebccfgd, reg | 0x04000000 );	/* Set ATC */

    mtebc( pb0ap, 0x03810480 );	/* DiskOnChip */
    mtebc( pb0cr, 0xfff18000 );
    
    /* PIC chip select line */    
    mtebc( pb1ap, 0x03810480 );	/* PIC */
    mtebc( pb1cr, 0xf0218000 );
    
    mtebc( pb2ap, 0x03810480 );	/* Xilinx FPGA */
    mtebc( pb2cr, 0xf0118000 );
    
    /* Legacy from openbios */
    mtebc( pb7ap, 0x01015280 );	/* FPGA registers */
    mtebc( pb7cr, 0x48318000 ); /* BA=0x483 1MB R/W 8-bit */

    /*--------------------------------------------------------------------
     * Setup the interrupt controller polarities, triggers, etc.
     *-------------------------------------------------------------------*/
    mtdcr( uic0sr, 0xffffffff );    /* clear all */
    mtdcr( uic0er, 0x00000000 );    /* disable all */
    mtdcr( uic0cr, 0x00000009 );    /* SMI & UIC1 crit are critical */
    mtdcr( uic0pr, 0xfffffe03 );    /* per ref-board manual */
    mtdcr( uic0tr, 0x01c00008 );    /* per ref-board manual */
    mtdcr( uic0vr, 0x00000001 );    /* int31 highest, base=0x000 */
    mtdcr( uic0sr, 0xffffffff );    /* clear all */
    
    mtdcr( uic1sr, 0xffffffff );    /* clear all */
    mtdcr( uic1er, 0x00000000 );    /* disable all */
    mtdcr( uic1cr, 0x00000000 );    /* all non-critical */
    mtdcr( uic1pr, 0xffffe0ff );    /* per ref-board manual */
    mtdcr( uic1tr, 0x00ffc000 );    /* per ref-board manual */
    mtdcr( uic1vr, 0x00000001 );    /* int31 highest, base=0x000 */
    mtdcr( uic1sr, 0xffffffff );    /* clear all */

    priority_setup();
    
    return 0;
}

void
misc_init_r(bd_t *bd)
{
    unsigned int *tavor_pci_config = (unsigned int *)CFG_PCI_BASE;
    unsigned int *tavor_pci_data = (unsigned int *)(CFG_PCI_BASE + 0x4);
    unsigned int *tavor_rebootsig_addr = (unsigned int *)TAVOR_REBOOTSIG_ADDR;
    unsigned int rddata, data;
    unsigned int tavor_rebootsig;
    int i;

    /* 
     * Check to find out if Tavor is a PCI device, or a memory controller.
     * If Tavor is a memory controller, try to reset it once. Perhaps Tavor
     * came out of reset before PIC could drive the FAD3 signal correctly.
     * 1 reset will workaround this situation. See bug #1872.
     */

    rddata = in32r(tavor_pci_config);
	data = TAVOR_CONFIG_TYPE;				/* bits 0:1 */
    data |= (PCI_VENDOR_ID << 2);			/* bits 7:2 */
	data |= (TAVOR_FUNC_NUM << 8);			/* bits 10:8 */
	data |= (TAVOR_DEV_NUM << 11);			/* bits 15:11 */
	data |= (TAVOR_BUS_NUM << 16);			/* bits 23:16 */
	data |= 0x80000000;						/* bit 31: enable */
	data |= (rddata & 0x7e000000);			/* bits 30:24 rsd */
	out32r(tavor_pci_config, data);
    rddata = in32r(tavor_pci_data);

    if ((rddata >> 16) == TAVOR_VENDID_PCI) {
        printf ("Tavor 00:01.0 PCI bridge: Mellanox Technology: %x\n", (rddata >> 16));
        printf ("POST: Tavor PCI Probe PASSED\n");
        sprintf(postresults, "%s TavorPOST=\"Tavor PCI Probe PASSED\" ", postresults);
    } else if ((rddata >> 16) == TAVOR_VENDID_MEM) {
        printf ("Tavor 00:01.0 Memory controller: Mellanox Technology: %x\n", (rddata >> 16));
        tavor_rebootsig = sread(tavor_rebootsig_addr);
        if (tavor_rebootsig != TAVOR_REBOOTSIG) {
            swrite(tavor_rebootsig_addr, TAVOR_REBOOTSIG);
            tavor_rebootsig = sread(tavor_rebootsig_addr);
            printf ("Reset ppcboot 1 time to see if Tavor comes up as a PCI device...\n");
            for (i=0; i<100; i++) ;
            do_reset(NULL,NULL,0,0,NULL);
        } else {
            swrite(tavor_rebootsig_addr, 0);
            tavor_rebootsig = sread(tavor_rebootsig_addr);
        }
    } else {
        printf ("Tavor 00:01.0 Unknown: %x\n", (rddata >> 16));
        printf ("POST: Tavor PCI Probe FAILED\n");
        sprintf(postresults, "%s TavorPOST=\"Tavor PCI Probe FAILED\" ", postresults);
    }
}


int checkboard (void)
{
    sys_info_t sysinfo;
    get_sys_info(&sysinfo);

    printf("Topspin FC Gateway\n");
    printf("\tVCO: %lu MHz\n", sysinfo.freqVCOMhz/1000000);
    printf("\tCPU: %lu MHz\n", sysinfo.freqProcessor/1000000);
    printf("\tPLB: %lu MHz\n", sysinfo.freqPLB/1000000);
    printf("\tOPB: %lu MHz\n", sysinfo.freqOPB/1000000);
    printf("\tEPB: %lu MHz\n", sysinfo.freqEPB/1000000);
    return (0);
}


long int initdram (int board_type)
{
    long    dram_size = 0;
#if defined(CONFIG_SPD_EEPROM)
    dram_size = spd_sdram();
#else
    dram_size = fixed_sdram();
#endif
  return dram_size;
}

int testsram(void)
{
    uint    *pstart = (uint *)CFG_ISRAM_BASE;
    uint    *p;

    /* 
     * The PPC440 has 8K of SRAM. Layout: can only write/read select locations.
     * The SP is using SRAM space, so if its not working, we probably didn't make 
     * here anyway.
     *
     *	0xc0000000
     *  0xc0000100 recovery image signature
     *  0xc0000110 Tavor reboot signature
     *  0xc0001f80 Initial SP
     *  0xc0001f84 - 0xc0002000 Init Data
     */

    p = pstart;
    *p = 0xaaaaaaaa;
    if( *p != 0xaaaaaaaa )
    {
        printf("POST: SRAM test fails at %08x FAILED\n", (uint)p );
        sprintf(postresults, "%s SRAMPOST=\"SRAM test fails at %08x FAILED\" ", postresults);
        return 1;
    }
    p = pstart;
    *p = 0x55555555;
    if( *p != 0x55555555 )
    {
        printf("POST: SRAM test fails at %08x FAILED\n", (uint)p );
        sprintf(postresults, "%s SRAMPOST=\"SRAM test fails at %08x FAILED\" ", postresults);
        return 1;
    }
    printf("POST: SRAM PASSED\n");
    sprintf(postresults, "%s SRAMPOST=\"SRAM PASSED\" ", postresults);
    return 0;
}

#if defined(CFG_DRAM_TEST)
int testdram(void)
{
    uint    *pstart = (uint *)0x00000000;
    uint    *pend   = (uint *)0x08000000;
    uint    *p;

    testsram();

    /* 
     * DRAM is mapped at 0 and is 128MB. Be careful which DRAM locations
     * we test at this point.
     */
 
    p = pend - 10;
    *p = 0xaaaaaaaa;
    if( *p != 0xaaaaaaaa )
    {
        printf("POST: DRAM test fails at %08x FAILED\n", (uint)p );
        sprintf(postresults, "%s DRAMPOST=\"DRAM test fails at %08x FAILED\" ", postresults);
        return 1;
    }
    p = pend - 10;
    *p = 0x55555555;
    if( *p != 0x55555555 )
    {
        printf("POST: DRAM test fails at %08x FAILED\n", (uint)p );
        sprintf(postresults, "%s DRAMPOST=\"DRAM test fails at %08x FAILED\" ", postresults);
        return 1;
    }
    printf("POST: DRAM PASSED\n");
    sprintf(postresults, "%s DRAMPOST=\"DRAM PASSED\" ", postresults);
    return 0;

#ifdef LONG
    for( p = pstart; p < pend; p++ )
        *p = 0xaaaaaaaa;

    for( p = pstart; p < pend; p++ )
    {
        if( *p != 0xaaaaaaaa )
        {
            printf("POST: DRAM test fails at %08x FAILED\n", (uint)p );
            return 1;
        }
    }

    for( p = pstart; p < pend; p++ )
        *p = 0x55555555;

    for( p = pstart; p < pend; p++ )
    {
        if( *p != 0x55555555 )
        {
            printf("POST: DRAM test fails at %08x FAILED\n", (uint)p );
            return 1;
        }
    }
    printf("POST: DRAM PASSED\n");
    return 0;
#endif
}
#endif

#if !defined(CONFIG_SPD_EEPROM)
/*************************************************************************
 *  fixed sdram init -- doesn't use serial presence detect.
 *
 *  Assumes:    128 MB, non-ECC, non-registered
 *              PLB @ 133 MHz
 *
 ************************************************************************/
long int fixed_sdram( void )
{
    return( 128 * 1024 * 1024 );    /* 128 MB                           */
}
#endif /* !defined(CONFIG_SPD_EEPROM) */
