/*
*  Copyright (C) 2002 Scott McNutt <smcnutt@artesyncp.com>
*
* See file CREDITS for list of people who contributed to this
* project.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston,
* MA 02111-1307 USA
*/


#include <ppcboot.h>
#include "ts_igr_fg.h"
#include <asm/processor.h>
#include <pci.h>

extern int swrite(volatile unsigned int *addr, unsigned int val);
extern unsigned int sread(volatile unsigned int *addr);

long int fixed_sdram( void );
void priority_setup(void);

int board_pre_init (void)
{
    uint reg;

    /*--------------------------------------------------------------------
     * GPIO map
     *
     * GPIO0 FIG INT0
     * GPIO1 FIG INT1
     * GPIO2 ADM TEMP INT
     * GPIO3 CPU FIG RESET GPIO
     * GPIO4 FIG FPGA PROG GPIO
     * GPIO5 CPU ENET PHY INT
     * GPIO6 PIC INT
     * GPIO7 CPU LVPGEN GPIO
     * GPIO8 PIC VPP GPIO
     * GPIO9 PIC CLK GPIO
     * GPIO10 PIC DATA GPIO
     * GPIO11 PIC PGM EN GPIO
     * GPIO12 UART1RX
     * GPIO13 UART1TX
     * GPIO14 FIG FPGA INIT GPIO
     * GPIO15 FIG FPGA DONE GPIO
     * GPIO16 I2C1SL (IBML CLK)
     * GPIO17 I2CSDA (IBML DATA)
     *-------------------------------------------------------------------*/

    /* Configure GPIO0-17 as listed above. Configure GPIO18-31 as gpios */
    mtdcr( cpc0_gpio, 0x19f30000 );

    /* Configure pin GPIO6 -> IRQ6, GPIO7-11 as GPIO, GPIO15 is GPIO */
    mtdcr( cpc0_gpio, 0x01f10000 );

    /* GPIO11 as output */
    out32(GPIO_TCR, in32(GPIO_TCR) | 0x10100000);

    /* Disable PIC programming */
    out32(GPIO_OR,  in32(GPIO_OR)  & ~0x00100000);

    /* Take FIG out of reset */
    out32(GPIO_OR, in32(GPIO_OR) | 0x10000000);

    /*--------------------------------------------------------------------
     * Setup the external bus controller/chip selects
     *-------------------------------------------------------------------*/
    mtdcr( ebccfga, xbcfg );
    reg = mfdcr( ebccfgd );
    mtdcr( ebccfgd, reg | 0x04000000 );	/* Set ATC */

    mtebc( pb0ap, 0x03814480 );	/* DiskOnChip WBN=1 increase WE setup time */
    mtebc( pb0cr, 0xfff18000 ); /* BA=0xfff 128MB R/W 8bit */
    
    /* PIC chip select line */    
    mtebc( pb1ap, 0x03810480 );	/* PIC */
    mtebc( pb1cr, 0xf0218000 );
    
    mtebc( pb2ap, 0x00000180 );	/* Xilinx FPGA FIG_HCS RE=1, SOR=1 */
    mtebc( pb2cr, 0xf011e000 ); /* 1 MB of 32 bit RW */

    /* 
     * FIG Xilinx workaround: cfg space data lines connected wrong, so
     * configure as a 32bit device and we'll write to address 3. The
     * FIG .hex file will swizzle the bits.
     */

    mtebc( pb3ap, 0x00000000 );	/* Xilinx FPGA FIG_CFG */
    mtebc( pb3cr, 0xf031e000 ); /* 1 MB of 32 bit RW */

    /* Legacy from openbios */
    mtebc( pb7ap, 0x01015280 );	/* FPGA registers */
    mtebc( pb7cr, 0x48318000 ); /* BA=0x483 1MB R/W 8-bit */

    /*--------------------------------------------------------------------
     * Setup the interrupt controller polarities, triggers, etc.
     *-------------------------------------------------------------------*/
    mtdcr( uic0sr, 0xffffffff );    /* clear all */
    mtdcr( uic0er, 0x00000000 );    /* disable all */
    mtdcr( uic0cr, 0x00000009 );    /* SMI & UIC1 crit are critical */
    mtdcr( uic0pr, 0xfffffe03 );    /* per ref-board manual */
    mtdcr( uic0tr, 0x01c00008 );    /* per ref-board manual */
    mtdcr( uic0vr, 0x00000001 );    /* int31 highest, base=0x000 */
    mtdcr( uic0sr, 0xffffffff );    /* clear all */
    
    mtdcr( uic1sr, 0xffffffff );    /* clear all */
    mtdcr( uic1er, 0x00000000 );    /* disable all */
    mtdcr( uic1cr, 0x00000000 );    /* all non-critical */
    mtdcr( uic1pr, 0xffffe0ff );    /* per ref-board manual */
    mtdcr( uic1tr, 0x00ffc000 );    /* per ref-board manual */
    mtdcr( uic1vr, 0x00000001 );    /* int31 highest, base=0x000 */
    mtdcr( uic1sr, 0xffffffff );    /* clear all */

    priority_setup();

    return 0;
}

void
misc_init_r(bd_t *bd)
{
}

int checkboard (void)
{
    sys_info_t sysinfo;
    get_sys_info(&sysinfo);

    printf("Topspin Ethernet-6 Gateway\n");
    printf("\tVCO: %lu MHz\n", sysinfo.freqVCOMhz/1000000);
    printf("\tCPU: %lu MHz\n", sysinfo.freqProcessor/1000000);
    printf("\tPLB: %lu MHz\n", sysinfo.freqPLB/1000000);
    printf("\tOPB: %lu MHz\n", sysinfo.freqOPB/1000000);
    printf("\tEPB: %lu MHz\n", sysinfo.freqEPB/1000000);
    return (0);
}


long int initdram (int board_type)
{
    long    dram_size = 0;
#if defined(CONFIG_SPD_EEPROM)
    dram_size = spd_sdram();
#else
    dram_size = fixed_sdram();
#endif
  return dram_size;
}


#if defined(CFG_DRAM_TEST)
int testdram(void)
{
    uint    *pstart = (uint *)0x00000000;
    uint    *pend   = (uint *)0x08000000;
    uint    *p;

    for( p = pstart; p < pend; p++ )
        *p = 0xaaaaaaaa;

    for( p = pstart; p < pend; p++ )
    {
        if( *p != 0xaaaaaaaa )
        {
            printf("SDRAM test fails at: %08x\n", (uint)p );
            return 1;
        }
    }

    for( p = pstart; p < pend; p++ )
        *p = 0x55555555;

    for( p = pstart; p < pend; p++ )
    {
        if( *p != 0x55555555 )
        {
            printf("SDRAM test fails at: %08x\n", (uint)p );
            return 1;
        }
    }
    return 0;
}
#endif

#if !defined(CONFIG_SPD_EEPROM)
/*************************************************************************
 *  fixed sdram init -- doesn't use serial presence detect.
 *
 *  Assumes:    128 MB, non-ECC, non-registered
 *              PLB @ 133 MHz
 *
 ************************************************************************/
long int fixed_sdram( void )
{
    return( 128 * 1024 * 1024 );    /* 128 MB                           */
}
#endif /* !defined(CONFIG_SPD_EEPROM) */
