/**************************************************************************
*
*   Module:			PPCBoot/board/ts_igr_lt
*
*   Filename:       fan.h
*
*   Author:         Brian Tran
*
*   Created:        01/08/2003
*
*   Description:
*
*   Fan controller ADM1031 registers.
*
***************************************************************************
*
*                 Source Control System Information
*
*    $Id: fan.h,v 1.8 2003/02/07 02:19:33 sarah Exp $
*
***************************************************************************
*
*                       Modification History
*
*    $Log: fan.h,v $
*    Revision 1.8  2003/02/07 02:19:33  sarah
*    adjust fan speed delta value
*
*    Revision 1.7  2003/02/05 22:32:53  sarah
*    enable INT, FAN_FAILURE, tach limit
*
*    Revision 1.6  2003/01/24 23:57:35  sarah
*    change range to 80 per Mark H.
*
*    Revision 1.5  2003/01/24 23:21:15  sarah
*    fix LOCALTEMP setting
*
*    Revision 1.4  2003/01/24 23:03:13  sarah
*    additional defines for auto fan cntrl
*
*    Revision 1.3  2003/01/20 19:02:33  sarah
*    change PWM freq. for smoother operation
*
*    Revision 1.2  2003/01/15 01:44:42  sarah
*    additions for POST
*
*    Revision 1.1  2003/01/08 18:29:34  sarah
*    code to turn fans on
*
*
***************************************************************************
*                       (c) COPYRIGHT 2000, 2001, 2002
*                           TopSpin Communications, Inc.
*                           Palo Alto, CA
**************************************************************************/
#ifndef _FAN_H
#define _FAN_H


/* deine register offset */
#define ADM_CONFIG1_OFFSET		0x00
#define ADM_CONFIG2_OFFSET		0x01

#define ADM_STATUS1_OFFSET		0x02
#define ADM_STATUS2_OFFSET		0x03

#define ADM_FAN_CHAR1_OFFSET		0x20
#define ADM_FAN_CHAR2_OFFSET		0x21

#define ADM_FAN_SPEED_CONFIG_OFFSET	0x22
#define ADM_FAN_FILTER_OFFSET		0x23

#define ADM_LOCALTEMP_CONFIG_OFFSET	0x24
#define ADM_REMOTE1TEMP_CONFIG_OFFSET	0x25
#define ADM_REMOTE2TEMP_CONFIG_OFFSET	0x26

#define ADM_THERM_BEHAVE_OFFSET		0x3f

#define ADM_FAN1_SPEED_OFFSET		0x08
#define ADM_FAN2_SPEED_OFFSET		0x09

/* 2300 RPM (10%) is the low limit = 0x24 */
#define ADM_FAN1_TACHHI_LIMIT_OFFSET 0x10
#define ADM_FAN2_TACHHI_LIMIT_OFFSET 0x11
#define ADM_TACHHI_LIMIT            0x24

#define ADM_LOCALTEMP_VALUE_OFFSET	0x0a
#define ADM_REMOTE1TEMP_VALUE_OFFSET	0x0b
#define ADM_REMOTE2TEMP_VALUE_OFFSET	0x0c

#define ADM_LOCAL_THERM_LIMIT_OFFSET 0x16

/* default config value */
#define ADM_CONFIG1_DEFAULT		0x1A
#define ADM_CONFIG1_DEFAULT_AUTO		0xFA
#define ADM_CONFIG2_DEFAULT		0x1f

/* interesting fields in the status registers */
#define ADM_ALARM_FAN_SPEED		0x01		/* bit 1 */
#define ADM_SLOW_FAN_SPEED		0x02		/* bit 2 */
#define ADM_HIGH_LOCALTEMP		0x40		/* bit 6 */

/* define default fan characteristics. Range N = 4: min speed to be measured is 662 RPM. Nominal speed is 2200 RPM */ 
#define ADM_FAN_CHAR_DEFAULT		0xfd

/* define fan filter default. Enable filter */
#define ADM_FAN_FILTER_DEFAULT		0x53


/* define THERM behavior: disable Therm-to-fan */
#define ADM_THERM_BEHAVE_DEFAULT	0x00


/* define fan speed measuring parameters */
#define ADM_FAN_SPEED_MEASURE_FREQ		11250		/* 11.25 KHz */
#define ADM_FAN_SPEED_RANGE				8
#define ADM_FAN_SPEED_MAX				9500
#define ADM_FAN_SPEED_MIN				3300
#define ADM_FAN_SPEED_DELTA				1500

#define ADM_FAN_SPEED_MAX_SETTING	 	0xff		/* 100% speed */
#define ADM_FAN_SPEED_MID_SETTING	 	0x88		/* 53% speed */
#define ADM_FAN_SPEED_MIN_SETTING	 	0x33		/* 20% speed */

/* default local Tmin/range = 0 degree min, 80 degree range */
#define ADM_LOCALTEMP_CONFIG_DEFAULT	0x04

/* retry when reading fan count zero */
#define MAX_FAN_COUNT_RETRY 10

#define FAN_CONTROL_MANUAL				0
#define FAN_CONTROL_AUTO				1

#endif _FAN_H
