/*
*  Copyright (C) 2002 Scott McNutt <smcnutt@artesyncp.com>
*
* See file CREDITS for list of people who contributed to this
* project.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston,
* MA 02111-1307 USA
*/


#include <ppcboot.h>
#include "ts_igr_lt.h"
#include <asm/processor.h>


long int fixed_sdram( void );
void priority_setup(void);

extern int dontboot;

int board_pre_init (void)
{
    uint reg;
    
    /*--------------------------------------------------------------------
     * GPIO map
     *
     * GPIO0 PCI ANA1 INTR
     * GPIO1 PCI ANA2 INTR
     * GPIO2 PCI ANA3 INTR
     * GPIO3 ENET PHY INTR
     * GPIO4 IM INTR
     * GPIO5 FPGA INTR
     * GPIO6 PS FAIL INTR
     * GPIO7 FPGA INIT OUTPUT
     * GPIO8 FPGA PROGRAM ENABLE OUTPUT
     * GPIO9 ANAFA1 RESET OUTPUT
     * GPIO10 ANAFA2 RESET OUTPUT
     * GPIO11 ANAFA3 RESET OUTPUT
     * GPIO12 UART1RX
     * GPIO13 UART1TX
     * GPIO14 FPGA DONE INPUT
     * GPIO15 TEMP/VOLT INTR
     * GPIO16 I2C1SL (IBML CLK)
     * GPIO17 I2CSDA (IBML DATA)
     *-------------------------------------------------------------------*/

    /* Configure GPIO0-17 as listed above. Configure GPIO18-31 as gpios */
    mtdcr( cpc0_gpio, 0x01f30000 );

    /*--------------------------------------------------------------------
     * Setup the external bus controller/chip selects
     *-------------------------------------------------------------------*/
    mtdcr( ebccfga, xbcfg );
    reg = mfdcr( ebccfgd );
    mtdcr( ebccfgd, reg | 0x04000000 );	/* Set ATC */

    mtebc( pb0ap, 0x03810480 );	/* DiskOnChip */
    mtebc( pb0cr, 0xfff18000 ); /* BA=0xfff 128MB R/W 8bit */
    
    /* Legacy from openbios */
    mtebc( pb1ap, 0x02815480 );	/* NVRAM/RTC */
    mtebc( pb1cr, 0x48018000 ); /* BA=0x480 1MB R/W 8-bit */
    
    mtebc( pb2ap, 0x00000200 );	/* Xilinx FPGA */
    mtebc( pb2cr, 0xf0118000 ); /* BA=0xf01 1MB R/W 8-bit */
    
    /* Legacy from openbios */
    mtebc( pb7ap, 0x01015280 );	/* FPGA registers */
    mtebc( pb7cr, 0x48318000 ); /* BA=0x483 1MB R/W 8-bit */

    /*--------------------------------------------------------------------
     * Setup the interrupt controller polarities, triggers, etc.
     *-------------------------------------------------------------------*/
    mtdcr( uic0sr, 0xffffffff );    /* clear all */
    mtdcr( uic0er, 0x00000000 );    /* disable all */
    mtdcr( uic0cr, 0x00000009 );    /* SMI & UIC1 crit are critical */
    mtdcr( uic0pr, 0xfffffe13 );    /* per ref-board manual */
    mtdcr( uic0tr, 0x01c00008 );    /* per ref-board manual */
    mtdcr( uic0vr, 0x00000001 );    /* int31 highest, base=0x000 */
    mtdcr( uic0sr, 0xffffffff );    /* clear all */
    
    mtdcr( uic1sr, 0xffffffff );    /* clear all */
    mtdcr( uic1er, 0x00000000 );    /* disable all */
    mtdcr( uic1cr, 0x00000000 );    /* all non-critical */
    mtdcr( uic1pr, 0xffffe0ff );    /* per ref-board manual */
    mtdcr( uic1tr, 0x00ffc000 );    /* per ref-board manual */
    mtdcr( uic1vr, 0x00000001 );    /* int31 highest, base=0x000 */
    mtdcr( uic1sr, 0xffffffff );    /* clear all */

    priority_setup();

    return 0;
}



int checkboard (void)
{
    sys_info_t sysinfo;
    get_sys_info(&sysinfo);

    printf("Topspin 90 Controller Card\n");
#ifdef DEBUG
    printf("\tVCO: %lu MHz\n", sysinfo.freqVCOMhz/1000000);
    printf("\tCPU: %lu MHz\n", sysinfo.freqProcessor/1000000);
    printf("\tPLB: %lu MHz\n", sysinfo.freqPLB/1000000);
    printf("\tOPB: %lu MHz\n", sysinfo.freqOPB/1000000);
    printf("\tEPB: %lu MHz\n", sysinfo.freqEPB/1000000);
#endif
    return (0);
}


long int initdram (int board_type)
{
    long    dram_size = 0;
#if defined(CONFIG_SPD_EEPROM)
    dram_size = spd_sdram();
#else
    dram_size = fixed_sdram();
#endif
  return dram_size;
}


#if defined(CFG_DRAM_TEST)
int testdram(void)
{
    uint    *pstart = (uint *)0x00000000;
    uint    *pend   = (uint *)0x08000000;
    uint    *p;

    for( p = pstart; p < pend; p++ )
        *p = 0xaaaaaaaa;

    for( p = pstart; p < pend; p++ )
    {
        if( *p != 0xaaaaaaaa )
        {
            printf("SDRAM test fails at: %08x\n", (uint)p );
            return 1;
        }
    }

    for( p = pstart; p < pend; p++ )
        *p = 0x55555555;

    for( p = pstart; p < pend; p++ )
    {
        if( *p != 0x55555555 )
        {
            printf("SDRAM test fails at: %08x\n", (uint)p );
            return 1;
        }
    }
    return 0;
}
#endif

#if !defined(CONFIG_SPD_EEPROM)
/*************************************************************************
 *  fixed sdram init -- doesn't use serial presence detect.
 *
 *  Assumes:    128 MB, non-ECC, non-registered
 *              PLB @ 133 MHz
 *
 ************************************************************************/
long int fixed_sdram( void )
{
    return( 128 * 1024 * 1024 );    /* 128 MB                           */
}
#endif /* !defined(CONFIG_SPD_EEPROM) */

extern int fpga_program(void);

void
misc_init_r(bd_t *bd)
{
    printf("FPGA loading...\n");
    fpga_program();

	/* After FPGA load, change bus timing to be like IGR CC */
    mtebc( pb2ap, 0x03810480 );	/* Xilinx FPGA */

    ledinit();
}

