
#include <ppcboot.h>
#include <command.h>
#include <cmd_sram.h>
#include <asm/processor.h>

#if defined(CONFIG_405GP)
unsigned int sread(volatile unsigned int *addr)
{
    return (*addr);
}
int swrite(volatile unsigned int *addr, unsigned int val)
{
    *addr = val;
    return 0;
}
#else
int swrite(volatile unsigned int *addr, unsigned int val)
{
    /* Avoid damaging other Internal SRAM areas */
    asm ("dcbi  0, %0" : : "r" (addr) );

    /* Set the memory location */
    *addr = val; /* some signature */

    /* flush the data cache */
    asm ("dcbf  0, %0" : : "r" (addr) );

    return 0;
}

unsigned int sread(volatile unsigned int *addr)
{
    /* Invalidate the data cache */
    asm ("dcbi  0, %0" : : "r" (addr) );

    return (*addr);
}
#endif

int do_sram (cmd_tbl_t *cmdtp, bd_t *bd, int flag, int argc, char *argv[])
{
  volatile unsigned int *addr, val;

  if (argc < 2 || argc > 4) {
    printf ("Usage:\n%s\n", cmdtp->usage);
    return 1;
  }

  addr = (unsigned int *) simple_strtoul(argv[2], NULL, 16);
  if (strcmp(argv[1], "read") == 0) {
      val = sread(addr);
      printf("sram 0x%x = %#x\n", addr, val);
  } else if (strcmp(argv[1], "write") == 0) {
      val = (unsigned int) simple_strtoul(argv[3], NULL, 16);
      printf ("sram write 0x%x = 0x%x\n", addr, val);
      return (swrite(addr, val));
  }
  return 0;
}

