/*----------------------------------------------------------------------------+
|
|  File Name:   440gp_pci.c
|
|  Function:    Initialization code for the 440GP PCI Configuration regs.
|
|  Author:      Sarah Rosedahl
|
|  Change Activity-
|
|  Date        Description of Change                                       BY
|  ---------   ---------------------                                       ---
|  16-May-03   Created                                                     SJR
+----------------------------------------------------------------------------*/

#include <ppcboot.h>
#include <command.h>
#include <cmd_boot.h>
#include <440gp_pci.h>
#include <asm/processor.h>
#include <pci.h>

#if defined(CONFIG_440)
#ifdef CONFIG_PCI

/*-----------------------------------------------------------------------------+
| pci_init.  Initializes the 405GP PCI Configuration regs.
|
| This code currently only does minimal configuration so that we can read
| Tavor config space to find out if Tavor came up as a PCI device, or as
| a memory controller.
+-----------------------------------------------------------------------------*/

void pci_440gp_init(void)
{
  int i, reg_num = 0;
  unsigned short temp_short;
  void *pcix_reg_base = PCIX0_REG_BASE;

        /* Set up Bridge Options 1 register like openbios */
        {
          u32 brdgopt1;

          brdgopt1 = in32r(pcix_reg_base + PCIX0_BRDGOPT1);
          brdgopt1 &= ~(0xffff0006);
          brdgopt1 |= 0x10000078;
          brdgopt1 &= ~(0x00000060);  /* Set PLB pri to 0 */ 
          brdgopt1 |= 0x10000000;
   
          out32r((pcix_reg_base + PCIX0_BRDGOPT1), brdgopt1);
        }

#ifdef DEBUG
        /* Write vendor id and read it for PCI bridge. For debug only. */
		{
		  u32 vendid;

 		  vendid = 0xefbeadde;
		  out32r((pcix_reg_base + PCIX0_VENDID), vendid);
 		  vendid = 0;
		  vendid = in32r((pcix_reg_base + PCIX0_VENDID));
		}
#endif

        /* Set bus 0, dev num 0 for PCI-X split transactions */
        {
          u32 pcixsts;

          pcixsts = in32r(pcix_reg_base + PCIX0_STS);
          pcixsts &= ~0xfff8;
          out32r((pcix_reg_base + PCIX0_STS), pcixsts);
        }
        /* Enable bus mastering and memory transactions */
        {
          u16 command;

          command = in16r(pcix_reg_base + PCIX0_COMMAND);
          command |= PCI_COMMAND_MASTER | PCI_COMMAND_MEMORY;
          out16r((pcix_reg_base + PCIX0_COMMAND), command);
        }

    /* Disable all windows */
    out32r(pcix_reg_base + PCIX0_POM0SA, 0);
    out32r(pcix_reg_base + PCIX0_POM1SA, 0);
    out32r(pcix_reg_base + PCIX0_POM2SA, 0);
    out32r(pcix_reg_base + PCIX0_PIM0SA, 0);
    out32r(pcix_reg_base + PCIX0_PIM1SA, 0);
    out32r(pcix_reg_base + PCIX0_PIM2SA, 0);

    /* Setup 2GB PLB->PCI outbound mem window (3_8000_0000->0_8000_0000) */
    out32r(pcix_reg_base + PCIX0_POM0LAH, 0x00000003);
    out32r(pcix_reg_base + PCIX0_POM0LAL, 0x80000000);
    out32r(pcix_reg_base + PCIX0_POM0PCIAH,0x00000000);
    out32r(pcix_reg_base + PCIX0_POM0PCIAL, 0x80000000);
    out32r(pcix_reg_base + PCIX0_POM0SA, 0x80000001);

    /* Setup 2GB PCI->PLB inbound memory window at 0, enable MSIs */
    out32r(pcix_reg_base + PCIX0_PIM0LAH, 0x00000000);
    out32r(pcix_reg_base + PCIX0_PIM0LAL, 0x00000000);
    out32r(pcix_reg_base + PCIX0_PIM0SA, 0xe0000007);

	/* Registers set in linux... */
    out32r(pcix_reg_base + PCIX0_BAR0L, 0x0000000c);
}

void pci_init(bd_t *bd)
{
  pci_440gp_init();
}

#endif /* CONFIG_PCI */
#endif /* CONFIG_405GP */
