/* m4/cpc.inc, openbios_ebony, ebony_bios 7/9/00 14:37:42 
!------------------------------------------------------------------------------+
!
!       This source code has been made available to you by IBM on an AS-IS
!       basis.  Anyone receiving this source is licensed under IBM
!       copyrights to use it in any way he or she deems fit, including
!       copying it, modifying it, compiling it, and redistributing it either
!       with or without modifications.  No license under IBM patents or
!       patent applications is to be implied by the copyright license.
!
!       Any user of this software should understand that IBM cannot provide
!       technical support for this software and will not be responsible for
!       any consequences resulting from the use of this software.
!
!       Any person who transfers this source code or any derivative work
!       must include the IBM copyright notice, this paragraph, and the
!       preceding two paragraphs in the transferred software.
!
!       COPYRIGHT   I B M   CORPORATION 1995
!       LICENSED MATERIAL  -  PROGRAM PROPERTY OF I B M
!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------
!
!  File Name:   cpc.inc
!
!  Function:    Defines for Clock, control, reset 
!		registers used in assembly language.
!
!  Author:      Neil Leeder
!
!  Change Activity-
!
!  Date        Description of Change                                       BY
!  ---------   ---------------------                                       ---
!  12-Apr-99   Created							   NL
!  16-May-00   New reg names    					   JWB
!  13-Sep-00   Port to 440GP                                               DWG
!
!-------------------------------------------------------------------------------
*/

#ifndef __CPC_INC__
#define __CPC_INC__

#ifdef __cplusplus
extern "C" {
#endif

#define CPC0_SYS0        (CPC0_DCR_BASE+0x0)  /* System Configuration Reg 0    */
#define CPC0_SYS1        (CPC0_DCR_BASE+0x1)  /* System Configuration Reg 1    */
#define CPC0_CUST0       (CPC0_DCR_BASE+0x2)  /* Customer Configuration Reg 0  */
#define CPC0_CUST1       (CPC0_DCR_BASE+0x3)  /* Customer Configuration Reg 1  */
#define CPC0_STRP0       (CPC0_DCR_BASE+0x4)  /* Power-On Configuration Reg 0  */
#define CPC0_STRP1       (CPC0_DCR_BASE+0x5)  /* Power-On Configuration Reg 1  */
#define CPC0_STRP2       (CPC0_DCR_BASE+0x6)  /* Power-On Configuration Reg 2  */
#define CPC0_STRP3       (CPC0_DCR_BASE+0x7)  /* Power-On Configuration Reg 3  */
#define CPC0_GPIO        (CPC0_DCR_BASE+0x8)  /* GPIO Configuration Reg        */
#define CPC0_PLB         (CPC0_DCR_BASE+0x9)  /* Power-On Configuration Reg    */
#define CPC0_CR1         (CPC0_DCR_BASE+0xA)  /* Control Register 1            */
#define CPC0_CR0         (CPC0_DCR_BASE+0xB)  /* Control Register 0            */
#define CPC0_MIRQ0       (CPC0_DCR_BASE+0xC)  /* Master Interrupt Req Reg 0    */
#define CPC0_MIRQ1       (CPC0_DCR_BASE+0xD)  /* Master Interrupt Req Reg 1    */
#define CPC0_RST         (CPC0_DCR_BASE+0xE)  /* Reset any individual core     */
#define CPC0_JTAGID      (CPC0_DCR_BASE+0xF)  /* JTAG ID Register              */

#define CPC0_SR          (POWERMAN_DCR_BASE+0x0) /* Power management status    */
#define CPC0_ER          (POWERMAN_DCR_BASE+0x1) /* Power management enable    */
#define CPC0_FR          (POWERMAN_DCR_BASE+0x2) /* Power management force     */

/* Defines for SYS0 register fields */
#define CPC0_SYS0_TUNE_MASK                  0xFFC00000
#define CPC0_SYS0_FBDV_MASK                  0x003C0000
#define   CPC0_SYS0_FBDV_16                  0x00000000
#define   CPC0_SYS0_FBDV_1                   0x00040000
#define   CPC0_SYS0_FBDV_2                   0x00080000
#define   CPC0_SYS0_FBDV_3                   0x000C0000
#define   CPC0_SYS0_FBDV_4                   0x00100000
#define   CPC0_SYS0_FBDV_5                   0x00140000
#define   CPC0_SYS0_FBDV_6                   0x00180000
#define   CPC0_SYS0_FBDV_7                   0x001C0000
#define   CPC0_SYS0_FBDV_8                   0x00200000
#define   CPC0_SYS0_FBDV_9                   0x00240000
#define   CPC0_SYS0_FBDV_10                  0x00280000
#define   CPC0_SYS0_FBDV_11                  0x002C0000
#define   CPC0_SYS0_FBDV_12                  0x00300000
#define   CPC0_SYS0_FBDV_13                  0x00340000
#define   CPC0_SYS0_FBDV_14                  0x00380000
#define   CPC0_SYS0_FBDV_15                  0x003C0000
#define CPC0_SYS0_FWDVA_MASK                 0x00038000
#define   CPC0_SYS0_FWDVA_8                  0x00000000
#define   CPC0_SYS0_FWDVA_7                  0x00008000
#define   CPC0_SYS0_FWDVA_6                  0x00010000
#define   CPC0_SYS0_FWDVA_5                  0x00018000
#define   CPC0_SYS0_FWDVA_4                  0x00020000
#define   CPC0_SYS0_FWDVA_3                  0x00028000
#define   CPC0_SYS0_FWDVA_2                  0x00030000
#define   CPC0_SYS0_FWDVA_1                  0x00038000
#define CPC0_SYS0_FWDVB_MASK                 0x00007000
#define   CPC0_SYS0_FWDVB_8                  0x00000000
#define   CPC0_SYS0_FWDVB_7                  0x00001000
#define   CPC0_SYS0_FWDVB_6                  0x00002000
#define   CPC0_SYS0_FWDVB_5                  0x00003000
#define   CPC0_SYS0_FWDVB_4                  0x00004000
#define   CPC0_SYS0_FWDVB_3                  0x00005000
#define   CPC0_SYS0_FWDVB_2                  0x00006000
#define   CPC0_SYS0_FWDVB_1                  0x00007000
#define CPC0_SYS0_OPDV_MASK                  0x00000C00
#define   CPC0_SYS0_OPDV_1                   0x00000000
#define   CPC0_SYS0_OPDV_2                   0x00000400
#define   CPC0_SYS0_OPDV_3                   0x00000800
#define   CPC0_SYS0_OPDV_4                   0x00000C00
#define CPC0_SYS0_EPDV_MASK                  0x00000300
#define   CPC0_SYS0_EPDV_1                   0x00000000
#define   CPC0_SYS0_EPDV_2                   0x00000100
#define   CPC0_SYS0_EPDV_3                   0x00000200
#define   CPC0_SYS0_EPDV_4                   0x00000300
#define CPC0_SYS0_EXTSL_MASK                 0x00000080
#define   CPC0_SYS0_EXTSL_CPU                0x00000000
#define   CPC0_SYS0_EXTSL_EBC                0x00000080
#define CPC0_SYS0_RW_MASK                    0x00000060
#define   CPC0_SYS0_RW_8                     0x00000000
#define   CPC0_SYS0_RW_16                    0x00000020
#define   CPC0_SYS0_RW_RESV                  0x00000040
#define   CPC0_SYS0_RW_32                    0x00000060
#define CPC0_SYS0_RL                         0x00000010
#define   CPC0_SYS0_RL_EBC                   0x00000000
#define   CPC0_SYS0_RL_PCI                   0x00000010
#define CPC0_SYS0_ZMIISL_MASK                0x0000000C
#define   CPC0_SYS0_ZMIISL_MII               0x00000000
#define   CPC0_SYS0_ZMIISL_SMII              0x00000004
#define   CPC0_SYS0_ZMIISL_RMII              0x00000008
#define   CPC0_SYS0_ZMIISL_RMII10            0x00000008
#define   CPC0_SYS0_ZMIISL_RMII100           0x0000000C 
#define CPC0_SYS0_BYPASS                     0x00000002
#define   CPC0_SYS0_BYPASS_DISABLED          0x00000000
#define   CPC0_SYS0_BYPASS_ENABLED           0x00000002
#define CPC0_SYS0_NTO1                       0x00000001

/* Defines for SYS1 register fields */
#define CPC0_SYS1_TUNE_MASK                  0xFFC00000
#define CPC0_SYS1_BYPASS                     0x00200000
#define   CPC0_SYS1_BYPASS_DISABLED          0x00000000
#define   CPC0_SYS1_BYPASS_ENABLED           0x00200000
#define CPC0_SYS1_PAE                        0x00100000
#define CPC0_SYS1_PHCE                       0x00080000
#define CPC0_SYS1_PISE                       0x00040000
#define CPC0_SYS1_PCWE                       0x00020000
#define CPC0_SYS1_PPIM                       0x0001E000
#define CPC0_SYS1_PR64E                      0x00001000
#define CPC0_SYS1_PXFS_MASK                  0x00000C00
#define   CPC0_SYS1_PXFS_100_133             0x00000000
#define   CPC0_SYS1_PXFS_66_100              0x00000400
#define   CPC0_SYS1_PXFS_50_66               0x00000800
#define CPC0_SYS1_PQXDE                      0x00000200
#define CPC0_SYS1_FLIPOFF                    0x00000100
#define CPC0_SYS1_TUNEEN                     0x00000040

/* Defines for CR0 register fields */
#define CPC0_CR0_SWE                         0x80000000
#define CPC0_CR0_CETE                        0x40000000
#define CPC0_CR0_U1FCS                       0x20000000
#define CPC0_CR0_U0DTE                       0x10000000
#define CPC0_CR0_U0DRE                       0x08000000
#define CPC0_CR0_U0DC                        0x04000000
#define CPC0_CR0_U1DTE                       0x02000000
#define CPC0_CR0_U1DRE                       0x01000000
#define CPC0_CR0_U1DC                        0x00800000
#define CPC0_CR0_U0EC                        0x00400000
#define CPC0_CR0_U1EC                        0x00200000
#define CPC0_CR0_UDIV_MASK                   0x001F0000
#define CPC0_CR0_UART_ENCODE(x) (((x)&0x1F)<<16)
#define CPC0_CR0_UART_DECODE(x) (((x)>>16)&0x1F)

/* Defines for CR1 register fields */
#define CPC0_CR1_DRU_MASK                    0xC0000000
#define CPC0_CR1_DRU_0                       0x00000000
#define CPC0_CR1_DRU_1                       0x40000000
#define CPC0_CR1_DRU_2                       0x80000000
#define CPC0_CR1_DRU_3                       0xC0000000
#define CPC0_CR1_DRT_MASK                    0x30000000
#define CPC0_CR1_DRT_0                       0x00000000
#define CPC0_CR1_DRT_1                       0x10000000
#define CPC0_CR1_DRT_2                       0x20000000
#define CPC0_CR1_DRT_3                       0x30000000
#define CPC0_CR1_DRNC_MASK                   0x0C000000
#define CPC0_CR1_DRNC_0                      0x00000000
#define CPC0_CR1_DRNC_1                      0x04000000
#define CPC0_CR1_DRNC_2                      0x08000000
#define CPC0_CR1_DRNC_3                      0x0C000000
#define CPC0_CR1_DRLC_MASK                   0x03000000
#define CPC0_CR1_DRLC_0                      0x00000000
#define CPC0_CR1_DRLC_1                      0x01000000
#define CPC0_CR1_DRLC_2                      0x02000000
#define CPC0_CR1_DRLC_3                      0x03000000
#define CPC0_CR1_DWF_MASK                    0x00C00000
#define CPC0_CR1_DWF_0                       0x00000000
#define CPC0_CR1_DWF_1                       0x00400000
#define CPC0_CR1_DWF_2                       0x00800000
#define CPC0_CR1_DWF_3                       0x00C00000
#define CPC0_CR1_DWS_MASK                    0x00300000
#define CPC0_CR1_DWS_0                       0x00000000
#define CPC0_CR1_DWS_1                       0x00100000
#define CPC0_CR1_DWS_2                       0x00200000
#define CPC0_CR1_DWS_3                       0x00300000
#define CPC0_CR1_DWU_MASK                    0x000C0000
#define CPC0_CR1_DWU_0                       0x00000000
#define CPC0_CR1_DWU_1                       0x00040000
#define CPC0_CR1_DWU_2                       0x00080000
#define CPC0_CR1_DWU_3                       0x000C0000
#define CPC0_CR1_IRF_MASK                    0x00030000
#define CPC0_CR1_IRF_0                       0x00000000
#define CPC0_CR1_IRF_1                       0x00010000
#define CPC0_CR1_IRF_2                       0x00020000
#define CPC0_CR1_IRF_3                       0x00030000
#define CPC0_CR1_IRT_MASK                    0x0000C000
#define CPC0_CR1_IRT_0                       0x00000000
#define CPC0_CR1_IRT_1                       0x00004000
#define CPC0_CR1_IRT_2                       0x00008000
#define CPC0_CR1_IRT_3                       0x0000C000
#define CPC0_CR1_IRS_MASK                    0x00003000
#define CPC0_CR1_IRS_0                       0x00000000
#define CPC0_CR1_IRS_1                       0x00001000
#define CPC0_CR1_IRS_2                       0x00002000
#define CPC0_CR1_IRS_3                       0x00003000
#define CPC0_CR1_TUNE_MASK                   0x0000003F
#define CPC0_CR1_TUNE_ENCODE(x) (((x)&0x3F)<<0)
#define CPC0_CR1_TUNE_DECODE(x) (((x)>>0)&0x3F)

/* Defines for the PLB register fields */
#define CPC0_PLB_APM0_MASK                   0xC0000000
#define CPC0_PLB_APM0_0                      0x00000000
#define CPC0_PLB_APM0_1                      0x40000000
#define CPC0_PLB_APM0_2                      0x80000000
#define CPC0_PLB_APM0_3                      0xC0000000
#define CPC0_PLB_APM1_MASK                   0x30000000
#define CPC0_PLB_APM1_0                      0x00000000
#define CPC0_PLB_APM1_1                      0x10000000
#define CPC0_PLB_APM1_2                      0x20000000
#define CPC0_PLB_APM1_3                      0x30000000
#define CPC0_PLB_APM2_MASK                   0x0C000000
#define CPC0_PLB_APM2_0                      0x00000000
#define CPC0_PLB_APM2_1                      0x04000000
#define CPC0_PLB_APM2_2                      0x08000000
#define CPC0_PLB_APM2_3                      0x0C000000
#define CPC0_PLB_APM3_MASK                   0x03000000
#define CPC0_PLB_APM3_0                      0x00000000
#define CPC0_PLB_APM3_1                      0x01000000
#define CPC0_PLB_APM3_2                      0x02000000
#define CPC0_PLB_APM3_3                      0x03000000
#define CPC0_PLB_APM5_MASK                   0x00300000
#define CPC0_PLB_APM5_0                      0x00000000
#define CPC0_PLB_APM5_1                      0x00100000
#define CPC0_PLB_APM5_2                      0x00200000
#define CPC0_PLB_APM5_3                      0x00300000
#define CPC0_PLB_APM6_MASK                   0x000C0000
#define CPC0_PLB_APM6_0                      0x00000000
#define CPC0_PLB_APM6_1                      0x00040000
#define CPC0_PLB_APM6_2                      0x00080000
#define CPC0_PLB_APM6_3                      0x000C0000
#define CPC0_PLB_APM7_MASK                   0x00030000
#define CPC0_PLB_APM7_0                      0x00000000
#define CPC0_PLB_APM7_1                      0x00010000
#define CPC0_PLB_APM7_2                      0x00020000
#define CPC0_PLB_APM7_3                      0x00030000
#define CPC0_PLB_M0PS_MASK                   0x00008000
#define CPC0_PLB_M0PS_MASTER                 0x00000000
#define CPC0_PLB_M0PS_ALT                    0x00008000
#define CPC0_PLB_M1PS_MASK                   0x00004000
#define CPC0_PLB_M1PS_MASTER                 0x00000000
#define CPC0_PLB_M1PS_ALT                    0x00004000
#define CPC0_PLB_M2PS_MASK                   0x00002000
#define CPC0_PLB_M2PS_MASTER                 0x00000000
#define CPC0_PLB_M2PS_ALT                    0x00002000
#define CPC0_PLB_M3PS_MASK                   0x00001000
#define CPC0_PLB_M3PS_MASTER                 0x00000000
#define CPC0_PLB_M3PS_ALT                    0x00001000
#define CPC0_PLB_M5PS_MASK                   0x00000400
#define CPC0_PLB_M5PS_MASTER                 0x00000000
#define CPC0_PLB_M5PS_ALT                    0x00000400
#define CPC0_PLB_M6PS_MASK                   0x00000200
#define CPC0_PLB_M6PS_MASTER                 0x00000000
#define CPC0_PLB_M6PS_ALT                    0x00000200
#define CPC0_PLB_M7PS_MASK                   0x00000100
#define CPC0_PLB_M7PS_MASTER                 0x00000000
#define CPC0_PLB_M7PS_ALT                    0x00000100
#define CPC0_PLB_SAP_MASK                    0x00000080
#define CPC0_PLB_SAP_DIS                     0x00000000
#define CPC0_PLB_SAP_ENAB                    0x00000080
#define CPC0_PLB_DAP_MASK                    0x00000040
#define CPC0_PLB_DAP_DIS                     0x00000000
#define CPC0_PLB_DAP_ENAB                    0x00000040
#define CPC0_PLB_POBAP_MASK                  0x00000020
#define CPC0_PLB_POBAP_DIS                   0x00000000
#define CPC0_PLB_POBAP_ENAB                  0x00000020

#define CPC0_CPM_IIC0       0x80000000
#define CPC0_CPM_IIC1       0x40000000
#define CPC0_CPM_PCI        0x20000000
#define CPC0_CPM_CPU        0x02000000
#define CPC0_CPM_DMA        0x01000000
#define CPC0_CPM_BGO        0x00800000
#define CPC0_CPM_BGI        0x00400000
#define CPC0_CPM_EBC        0x00200000
#define CPC0_CPM_EBM        0x00100000
#define CPC0_CPM_DMC        0x00080000
#define CPC0_CPM_PLB        0x00040000
#define CPC0_CPM_SRAM       0x00020000
#define CPC0_CPM_PPM        0x00002000
#define CPC0_CPM_UIC1       0x00001000
#define CPC0_CPM_GPIO       0x00000800
#define CPC0_CPM_GPT        0x00000400
#define CPC0_CPM_UART0      0x00000200
#define CPC0_CPM_UART1      0x00000100
#define CPC0_CPM_UIC0       0x00000080
#define CPC0_CPM_TMRCLK     0x00000040
#define CPC0_CPM_E0_MM      0x00000020
#define CPC0_CPM_E0_RM      0x00000010
#define CPC0_CPM_E0_TM      0x00000008
#define CPC0_CPM_E1_MM      0x00000004
#define CPC0_CPM_E1_RM      0x00000002
#define CPC0_CPM_E1_TM      0x00000001

#ifdef __cplusplus
}
#endif

#endif /* __CPC_INC__ */
