/* m4/pcix.inc, openbios_ebony, ebony_bios 7/9/00 14:37:48 
!------------------------------------------------------------------------------+
!
!       This source code has been made available to you by IBM on an AS-IS
!       basis.  Anyone receiving this source is licensed under IBM
!       copyrights to use it in any way he or she deems fit, including
!       copying it, modifying it, compiling it, and redistributing it either
!       with or without modifications.  No license under IBM patents or
!       patent applications is to be implied by the copyright license.
!
!       Any user of this software should understand that IBM cannot provide
!       technical support for this software and will not be responsible for
!       any consequences resulting from the use of this software.
!
!       Any person who transfers this source code or any derivative work
!       must include the IBM copyright notice, this paragraph, and the
!       preceding two paragraphs in the transferred software.
!
!       COPYRIGHT   I B M   CORPORATION 1995
!       LICENSED MATERIAL  -  PROGRAM PROPERTY OF I B M
!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------
!
!  File Name:   pcix.inc 
!
!  Function:    Defines for PCI-X interface.
!
!  Author:      Daniel Goehring
!
!  Change Activity-
!
!  Date        Description of Change                                       BY
!  ---------   ---------------------                                       ---
!  13-Sep-00   Created                                                     DWG
!
!-------------------------------------------------------------------------------
*/

#ifndef __PCIX_INC__
#define __PCIX_INC__

#ifdef __cplusplus
extern "C" {
#endif

/*----------------------------------------------------------------------------+
| Defines for the standard PCI device configuration header.
+----------------------------------------------------------------------------*/
#define PCIX_VENDID    0x00
#define PCIX_DEVID     0x02
#define PCIX_CMD       0x04
  #define PCIX_CMD_SERE     0x0100
  #define PCIX_CMD_PME      0x0004
  #define PCIX_CMD_MEMA     0x0002
  #define PCIX_CMD_IOA      0x0001
#define PCIX_STATUS    0x06
  #define PCIX_STATUS_66C   0x0020
#define PCIX_REVID     0x08
#define PCIX_CLS       0x09
  #define PCIX_CLS_INT  0x09
  #define PCIX_CLS_SUB  0x0A
  #define PCIX_CLS_BASE 0x0B
#define PCIX_CACHELS   0x0C
#define PCIX_LATTIM    0x0D
#define PCIX_HDTYPE    0x0E
#define PCIX_BIST      0x0F
#define PCIX_BAR0      0x10 
#define PCIX_BAR1      0x14 
#define PCIX_BAR2      0x18
#define PCIX_BAR3      0x1C
#define PCIX_BAR4      0x20
#define PCIX_BAR5      0x24 
#define PCIX_CISPTR    0x28
#define PCIX_SBSYSVID  0x2C
#define PCIX_SBSYSID   0x2E
#define PCIX_EROMBA    0x30
#define PCIX_CAP       0x34
#define PCIX_INTLN     0x3C
#define PCIX_INTPN     0x3D
#define PCIX_MINGNT    0x3E
#define PCIX_MAXLTNCY  0x3F
#define PCIX_BRDGOPT1  0x40
#define   PCIX_BRDGOPT1_RBPL7_MASK 0xC0000000
#define     PCIX_BRDGOPT1_RBPL7_0  0x00000000
#define     PCIX_BRDGOPT1_RBPL7_1  0x40000000
#define     PCIX_BRDGOPT1_RBPL7_2  0x80000000
#define     PCIX_BRDGOPT1_RBPL7_3  0xC0000000
#define   PCIX_BRDGOPT1_RBPL6_MASK 0x30000000
#define     PCIX_BRDGOPT1_RBPL6_0  0x00000000
#define     PCIX_BRDGOPT1_RBPL6_1  0x10000000
#define     PCIX_BRDGOPT1_RBPL6_2  0x20000000
#define     PCIX_BRDGOPT1_RBPL6_3  0x30000000
#define   PCIX_BRDGOPT1_RBPL5_MASK 0x0C000000
#define     PCIX_BRDGOPT1_RBPL5_0  0x00000000
#define     PCIX_BRDGOPT1_RBPL5_1  0x04000000
#define     PCIX_BRDGOPT1_RBPL5_2  0x08000000
#define     PCIX_BRDGOPT1_RBPL5_3  0x0C000000
#define   PCIX_BRDGOPT1_RBPL4_MASK 0x03000000
#define     PCIX_BRDGOPT1_RBPL4_0  0x00000000
#define     PCIX_BRDGOPT1_RBPL4_1  0x01000000
#define     PCIX_BRDGOPT1_RBPL4_2  0x02000000
#define     PCIX_BRDGOPT1_RBPL4_3  0x03000000
#define   PCIX_BRDGOPT1_RBPL3_MASK 0x00C00000
#define     PCIX_BRDGOPT1_RBPL3_0  0x00000000
#define     PCIX_BRDGOPT1_RBPL3_1  0x00400000
#define     PCIX_BRDGOPT1_RBPL3_2  0x00800000
#define     PCIX_BRDGOPT1_RBPL3_3  0x00C00000
#define   PCIX_BRDGOPT1_RBPL2_MASK 0x00300000
#define     PCIX_BRDGOPT1_RBPL2_0  0x00000000
#define     PCIX_BRDGOPT1_RBPL2_1  0x00100000
#define     PCIX_BRDGOPT1_RBPL2_2  0x00200000
#define     PCIX_BRDGOPT1_RBPL2_3  0x00300000
#define   PCIX_BRDGOPT1_RBPL1_MASK 0x000C0000
#define     PCIX_BRDGOPT1_RBPL1_0  0x00000000
#define     PCIX_BRDGOPT1_RBPL1_1  0x00040000
#define     PCIX_BRDGOPT1_RBPL1_2  0x00080000
#define     PCIX_BRDGOPT1_RBPL1_3  0x000C0000
#define   PCIX_BRDGOPT1_RBPL0_MASK 0x00030000
#define     PCIX_BRDGOPT1_RBPL0_0  0x00000000
#define     PCIX_BRDGOPT1_RBPL0_1  0x00010000
#define     PCIX_BRDGOPT1_RBPL0_2  0x00020000
#define     PCIX_BRDGOPT1_RBPL0_3  0x00030000
#define   PCIX_BRDGOPT1_PLREQ_MASK 0x00000060
#define     PCIX_BRDGOPT1_PLREQ_0  0x00000000
#define     PCIX_BRDGOPT1_PLREQ_1  0x00000020
#define     PCIX_BRDGOPT1_PLREQ_2  0x00000040
#define     PCIX_BRDGOPT1_PLREQ_3  0x00000060
#define   PCIX_BRDGOPT1_MSIEN      0x00000010
#define   PCIX_BRDGOPT1_PCIARB     0x00000008
#define   PCIX_BRDGOPT1_PCITMR_MASK 0x00000006
#define     PCIX_BRDGOPT1_PCITMR_MEMRD     0x00000000
#define     PCIX_BRDGOPT1_PCITMR_MEMRDLN   0x00000002
#define     PCIX_BRDGOPT1_PCITMR_MEMRDMULT 0x00000004
#define PCIX_BRDGOPT2  0x44        /* 440GP specific parameters */
#define   PCIX_BRDGOPT2_PCIFREQ   0x00018000
#define   PCIX_BRDGOPT2_PCIFREQ00 0x00000000
#define   PCIX_BRDGOPT2_PCIFREQ01 0x00008000
#define   PCIX_BRDGOPT2_PCIFREQ02 0x00010000
#define   PCIX_BRDGOPT2_PCIFREQ03 0x00018000
#define   PCIX_BRDGOPT2_PCIXENB   0x00004000
#define   PCIX_BRDGOPT2_PLBENBL   0x00001000
#define PCIX_ERREN     0x50    
#define PCIX_ERRSTS    0x54    
#define PCIX_PLBBESR   0x58
#define PCIX_PLBBEARL  0x5C    
#define PCIX_PLBBEARH  0x60    
#define PCIX_POM0LAL   0x68    
#define PCIX_POM0LAH   0x6C  
#define PCIX_POM0MA    0x70    
#define PCIX_POM0PCIAL 0x74    
#define PCIX_POM0PCIAH 0x78    
#define PCIX_POM1LAL   0x7C    
#define PCIX_POM1LAH   0x80    
#define PCIX_POM1MA    0x84    
#define PCIX_POM1PCIAL 0x88
#define PCIX_POM1PCIAH 0x8C
#define PCIX_POM2MA    0x90
#define PCIX_PIM0SA    0x98
#define PCIX_PIM0LAL   0x9C
#define PCIX_PIM0LAH   0xA0
#define PCIX_PIM1SA    0xA4
#define PCIX_PIM1LAL   0xA8
#define PCIX_PIM1LAH   0xAC
#define PCIX_PIM2SA    0xB0
#define PCIX_PIM2LAL   0xB4
#define PCIX_PIM2LAH   0xB8
#define PCIX_OMCAPID   0xC0
#define PCIX_OMNIPITR  0xC1
#define PCIX_OMMC      0xC2
#define PCIX_OMMA      0xC4
#define PCIX_OMMUA     0xC8
#define PCIX_OMMDATA   0xCC
#define PCIX_OMMEOI    0xCE
#define PCIX_PMCAPID   0xD0
#define PCIX_PMNIPTR   0xD1
#define PCIX_PMC       0xD2
#define PCIX_PMCSR     0xD4
#define PCIX_PMCSRBSE  0xD6
#define PCIX_PMDATA    0xD7
#define PCIX_PMSCRR    0xD8
#define PCIX_PCIXCAPID 0xDC
#define PCIX_PCIXNIPTR 0xDD
#define PCIX_PCIXCMD   0xDE
#define PCIX_PCIXSTS   0xE0
#define   PCIX_PCIXSTS_BUSNUM     0x0000FF00
#define   PCIX_PCIXSTS_DEVNUM     0x000000F8
#define PCIX_PCIXIDR   0xE4
#define PCIX_PCIXCID   0xE8
#define PCIX_PCIXRID   0xEC
#define PCIX_MSGIL     0x100
#define PCIX_MSGIH     0x104
#define PCIX_MSGOL     0x108
#define PCIX_MSGOH     0x10C
#define PCIX_IM        0x1F8

/*----------------------------------------------------------------------------+
| Defines for 440GP configuration regs.
+----------------------------------------------------------------------------*/
#define PCIX0_VENDID    (PCIX0_CFGBASE + PCIX_VENDID)
#define PCIX0_DEVID     (PCIX0_CFGBASE + PCIX_DEVID)
#define PCIX0_CMD       (PCIX0_CFGBASE + PCIX_CMD)
#define PCIX0_STATUS    (PCIX0_CFGBASE + PCIX_STATUS)
#define PCIX0_REVID     (PCIX0_CFGBASE + PCIX_REVID)
#define PCIX0_CLS       (PCIX0_CFGBASE + PCIX_CLS)
  #define PCIX0_INTCLS  (PCIX0_CFGBASE + PCIX_INTCLS)
  #define PCIX0_SUBCLS  (PCIX0_CFGBASE + PCIX_SUBCLS)
  #define PCIX0_BASECLS (PCIX0_CFGBASE + PCIX_BASECLS)
#define PCIX0_CACHELS   (PCIX0_CFGBASE + PCIX_CACHELS)
#define PCIX0_LATTIM    (PCIX0_CFGBASE + PCIX_LATTIM)
#define PCIX0_HDTYPE    (PCIX0_CFGBASE + PCIX_HDTYPE)
#define PCIX0_BIST      (PCIX0_CFGBASE + PCIX_BIST)
#define PCIX0_BAR0      (PCIX0_CFGBASE + PCIX_BAR0)
#define PCIX0_BAR1      (PCIX0_CFGBASE + PCIX_BAR1) 
#define PCIX0_BAR2      (PCIX0_CFGBASE + PCIX_BAR2)
#define PCIX0_BAR3      (PCIX0_CFGBASE + PCIX_BAR3)
#define PCIX0_BAR4      (PCIX0_CFGBASE + PCIX_BAR4)
#define PCIX0_BAR5      (PCIX0_CFGBASE + PCIX_BAR5)
#define PCIX0_CISPTR    (PCIX0_CFGBASE + PCIX_CISPTR)
#define PCIX0_SBSYSVID  (PCIX0_CFGBASE + PCIX_SBSYSVID)
#define PCIX0_SBSYSID   (PCIX0_CFGBASE + PCIX_SBSYSID)
#define PCIX0_EROMBA    (PCIX0_CFGBASE + PCIX_EROMBA)
#define PCIX0_CAP       (PCIX0_CFGBASE + PCIX_CAP)
#define PCIX0_INTLN     (PCIX0_CFGBASE + PCIX_INTLN)
#define PCIX0_INTPN     (PCIX0_CFGBASE + PCIX_INTPN)
#define PCIX0_MINGNT    (PCIX0_CFGBASE + PCIX_MINGNT)
#define PCIX0_MAXLTNCY  (PCIX0_CFGBASE + PCIX_MAXLTNCY)
#define PCIX0_BRDGOPT1  (PCIX0_CFGBASE + PCIX_BRDGOPT1)
#define PCIX0_BRDGOPT2  (PCIX0_CFGBASE + PCIX_BRDGOPT2)
#define PCIX0_ERREN     (PCIX0_CFGBASE + PCIX_ERREN)
#define PCIX0_ERRSTS    (PCIX0_CFGBASE + PCIX_ERRSTS)
#define PCIX0_PLBBESR   (PCIX0_CFGBASE + PCIX_PLBBESR)    
#define PCIX0_PLBBEARL  (PCIX0_CFGBASE + PCIX_PLBBEARL) 
#define PCIX0_PLBBEARH  (PCIX0_CFGBASE + PCIX_PLBBEARH) 
#define PCIX0_POM0LAL   (PCIX0_CFGBASE + PCIX_POM0LAL) 
#define PCIX0_POM0LAH   (PCIX0_CFGBASE + PCIX_POM0LAH) 
#define PCIX0_POM0MA    (PCIX0_CFGBASE + PCIX_POM0MA)
#define PCIX0_POM0PCIAL (PCIX0_CFGBASE + PCIX_POM0PCIAL) 
#define PCIX0_POM0PCIAH (PCIX0_CFGBASE + PCIX_POM0PCIAH)
#define PCIX0_POM1LAL   (PCIX0_CFGBASE + PCIX_POM1LAL)
#define PCIX0_POM1LAH   (PCIX0_CFGBASE + PCIX_POM1LAH)
#define PCIX0_POM1MA    (PCIX0_CFGBASE + PCIX_POM1MA)
#define PCIX0_POM1PCIAL (PCIX0_CFGBASE + PCIX_POM1PCIAL)
#define PCIX0_POM1PCIAH (PCIX0_CFGBASE + PCIX_POM1PCIAH)
#define PCIX0_POM2MA    (PCIX0_CFGBASE + PCIX_POM2MA)
#define PCIX0_PIM0SA    (PCIX0_CFGBASE + PCIX_PIM0SA)
#define PCIX0_PIM0LAL   (PCIX0_CFGBASE + PCIX_PIM0LAL)
#define PCIX0_PIM0LAH   (PCIX0_CFGBASE + PCIX_PIM0LAH)
#define PCIX0_PIM1SA    (PCIX0_CFGBASE + PCIX_PIM1SA)
#define PCIX0_PIM1LAL   (PCIX0_CFGBASE + PCIX_PIM1LAL)
#define PCIX0_PIM1LAH   (PCIX0_CFGBASE + PCIX_PIM1LAH)
#define PCIX0_PIM2SA    (PCIX0_CFGBASE + PCIX_PIM2SA)
#define PCIX0_PIM2LAL   (PCIX0_CFGBASE + PCIX_PIM2LAL)
#define PCIX0_PIM2LAH   (PCIX0_CFGBASE + PCIX_PIM2LAH)
#define PCIX0_OMCAPID   (PCIX0_CFGBASE + PCIX_OMCAPID)
#define PCIX0_OMNIPITR  (PCIX0_CFGBASE + PCIX_OMNIPITR)
#define PCIX0_OMMC      (PCIX0_CFGBASE + PCIX_OMMC)
#define PCIX0_OMMA      (PCIX0_CFGBASE + PCIX_OMMA)
#define PCIX0_OMMUA     (PCIX0_CFGBASE + PCIX_OMMUA)
#define PCIX0_OMMDATA   (PCIX0_CFGBASE + PCIX_OMMDATA)
#define PCIX0_OMMEOI    (PCIX0_CFGBASE + PCIX_OMMEOI)
#define PCIX0_PMCAPID   (PCIX0_CFGBASE + PCIX_PMCAPID)
#define PCIX0_PMNIPTR   (PCIX0_CFGBASE + PCIX_PMNIPTR)
#define PCIX0_PMC       (PCIX0_CFGBASE + PCIX_PMC)
#define PCIX0_PMCSR     (PCIX0_CFGBASE + PCIX_PMCSR)
#define PCIX0_PMCSRBSE  (PCIX0_CFGBASE + PCIX_PMCSRBSE)
#define PCIX0_PMDATA    (PCIX0_CFGBASE + PCIX_PMDATA)
#define PCIX0_PMSCRR    (PCIX0_CFGBASE + PCIX_PMSCRR)
#define PCIX0_PCIXCAPID (PCIX0_CFGBASE + PCIX_PCIXCAPID)
#define PCIX0_PCIXNIPTR (PCIX0_CFGBASE + PCIX_PCIXNIPTR)
#define PCIX0_PCIXCMD   (PCIX0_CFGBASE + PCIX_PCIXCMD)
#define PCIX0_PCIXSTS   (PCIX0_CFGBASE + PCIX_PCIXSTS)
#define PCIX0_PCIXIDR   (PCIX0_CFGBASE + PCIX_PCIXIDR)
#define PCIX0_PCIXCID   (PCIX0_CFGBASE + PCIX_PCIXCID)
#define PCIX0_PCIXRID   (PCIX0_CFGBASE + PCIX_PCIXRID)
#define PCIX0_MSGIL     (PCIX0_CFGBASE + PCIX_MSGIL)
#define PCIX0_MSGIH     (PCIX0_CFGBASE + PCIX_MSGIH)
#define PCIX0_MSGOL     (PCIX0_CFGBASE + PCIX_MSGOL)
#define PCIX0_MSGOH     (PCIX0_CFGBASE + PCIX_MSGOH)
#define PCIX0_IM        (PCIX0_CFGBASE + PCIX_IM)

#ifdef __cplusplus
}
#endif

#endif /* __PCIX_INC__ */
